/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                        OS_186.H                          *
*                80186/80188 Specific Code                 *
*                    SMALL MEMORY MODEL                    *
***********************************************************/
#ifndef _OS_186_
 #define _OS_186_

/* no compiler libraries are used */

 #define FP_SEG(fp) (*((unsigned __far *)&(fp)+1))
 #define FP_OFF(fp) (*((unsigned __far *)&(fp)))

// #define REAL_ADR(fp) ((((unsigned long)(fp)&0xffff0000)>>12)+((unsigned long)(fp)&0x0000ffff)) 
// #define NORM_ADR(fp) ((((unsigned long)(fp)&0xfffffff0)<<12)+((unsigned long)(fp)&0x0000000f))

 #define FARPTR(seg,off) (void _far*)((unsigned long)(seg)<<16|(off))

 #define  PCOS  0x12      /* Context switch vector number */

/*
************************************************************
*                        MACROS
************************************************************
*/
 #define  OS_CRITICAL_SR       0                             // do not save the CPU-register during INT-disable

 #define  OS_ENTER_CRITICAL()  _asm{ CLI  }
 #define  OS_EXIT_CRITICAL()   _asm{ STI  }
 #define  OS_TASK_SW()         _asm{ INT PCOS }
/*
************************************************************
*                    ENDIAN CONVENTION
************************************************************
*/
 #define  LITTLE_ENDIAN   3412
 #define  BIG_ENDIAN      1234

 #define  BYTE_ORDER  LITTLE_ENDIAN   /* endianess of CPU */

//------------ for little-endian protocols -----------------
 #if BYTE_ORDER == LITTLE_ENDIAN                           // for example: USB 
  #define L_htonl(n)       (n)
  #define L_htons(n)       (n)
 #else
  #define L_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define L_htons(n)       (((n & 0xff00U) >> 8) | ((n & 0x00ffU) << 8))
 #endif
 #define  L_ntohl          L_htonl
 #define  L_ntohs          L_htons
//-------------- for big-endian protocols ------------------
 #if BYTE_ORDER == BIG_ENDIAN                              // for example: 802.3 , TCP/IP
  #define B_htonl(n)       (n)
  #define B_htons(n)       (n)
 #else
  #define B_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define B_htons(n)       (((n & 0xff00U) >> 8) | ((n & 0x00ffU) << 8))
 #endif
 #define  B_ntohl          B_htonl
 #define  B_ntohs          B_htons
/*
************************************************************
*                 POINTER AND REENTRANCE
************************************************************
*/
 #define  OS_EXT  extern   /* define external declaration */
 #define  OS_FAR  far         /* define OS_FAR            */
 #define  OS_HUGE huge        /* define OS_HUGE for segm. */
 #define  OS_ROMDATA _based(_segname("_CODE"))
 #define  KC51_REENTRANT                    /* not needed */
 #define  SET_VECTOR(num,fun) *((U32 far *)(num*2))=(U32)(fun)
/*
************************************************************
*                       DATA TYPES
************************************************************
*/
 typedef unsigned char  BOOLEAN;
 typedef unsigned char  U08;
 typedef signed   char  S08;
 typedef unsigned short U16;
 typedef signed   short S16;
 typedef unsigned long  U32;
 typedef signed   long  S32;
 typedef float          FP;
 typedef U16            OS_STK_TYPE;

/*
************************************************************
*                       CONSTANTS
************************************************************
*/
 #ifndef FALSE
  #define FALSE  0
 #endif
 #ifndef TRUE
  #define TRUE   1
 #endif
 #ifndef NULL
  #define NULL   ((void *)0)
 #endif

/*
************************************************************
*                  COMPILING DEFINITIONS
************************************************************
*/
#ifndef OS_PACKED
 #define OS_PACKED
#endif
#ifndef OS_PACKED_ATTR
 #define OS_PACKED_ATTR
#endif
#ifndef OS_ALIGN_ATTR
 #define OS_ALIGN_ATTR
#endif

/************************** END ***************************/
#endif
