/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                     OS_AT91SAM3xxx.h                     *
*        Atmel Cortex-M3 AT91SAM3xxx Specific Code         *
*                                                          *
***********************************************************/
#ifndef _OS_AT91SAM3xxx_
 #define _OS_AT91SAM3xxx_

/*
************************************************************
*                  COMPILING DEFINITIONS
************************************************************
*/
 #define OS_CRITICAL_SR      1                             // save the CPU-register during INT-disable

 #ifdef __IMAGECRAFT__                                     // ICC compilers
  #define OS_PACK_PRAGMA                                   // - use "#pragma" (can not encapsulated)
 #else
  #ifdef __IAR_SYSTEMS_ICC__                               // IAR compilers
   #define OS_PACK_PRAGMA                                  // - use "#pragma"
   #define _NOP() __asm("nop")
  #else
   #ifdef __KEIL__                                         // old KEIL compilers
    #define OS_PACK_PRAGMA                                 // - use "#pragma"
   #else
    #ifdef __GNUC__                                        // GCC compilers
     #define OS_PACKED_ATTR __attribute__ ((__packed__))
     #define OS_ALIGN_ATTR  __attribute__ ((aligned(4)))
     #define _NOP()         __asm("nop")
    #else
     #ifdef __CC_ARM                                       // ADS/new Keil compilers
      #define OS_PACKED     __packed
      #define OS_ALIGN_ATTR __align(4)
      #define _NOP()        __asm {nop;}
     #else
      #error "unknown compiler"
     #endif
    #endif
   #endif
  #endif
 #endif

 #ifndef OS_PACKED
  #define OS_PACKED
 #endif
 #ifndef OS_PACKED_ATTR
  #define OS_PACKED_ATTR
 #endif
 #ifndef OS_ALIGN_ATTR
  #define OS_ALIGN_ATTR
 #endif

/*
************************************************************
*                    ENDIAN CONVENTION
************************************************************
*/
 #define  LITTLE_ENDIAN   3412
 #define  BIG_ENDIAN      1234

 #define  BYTE_ORDER  LITTLE_ENDIAN   /* endianess of CPU */

//------------ for little-endian protocols -----------------
 #if BYTE_ORDER == LITTLE_ENDIAN                           // for example: USB
  #define L_htonl(n)       n
  #define L_htons(n)       n
 #else
  #define L_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define L_htons(n)       (((n & 0xff00U) >> 8) | ((U16)(n & 0x00ffU) << 8))
 #endif
 #define  L_ntohl          L_htonl
 #define  L_ntohs          L_htons
//-------------- for big-endian protocols ------------------
 #if BYTE_ORDER == BIG_ENDIAN                              // for example: 802.3 , TCP/IP
  #define B_htonl(n)       n
  #define B_htons(n)       n
 #else
  #define B_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define B_htons(n)       (((n & 0xff00U) >> 8) | ((U16)(n & 0x00ffU) << 8))
 #endif
 #define  B_ntohl          B_htonl
 #define  B_ntohs          B_htons

/*
************************************************************
*                 POINTER AND REENTRANCE
************************************************************
*/
 #define  OS_EXT  extern   /* define external declaration */
 #define  OS_FAR              /* not needed               */
 #define  OS_HUGE             /* define OS_HUGE           */
 #define  OS_CONST   const    /* constant data            */
 #define  OS_ROMDATA          /* area locations in ROM    */
 #define  KC51_REENTRANT                    /* not needed */

/*
************************************************************
*                      DATA TYPES
************************************************************
*/
 typedef unsigned char  BOOLEAN;
 typedef unsigned char  U08;
 typedef signed   char  S08;
 typedef unsigned short U16;
 typedef signed   short S16;
 typedef unsigned int   U32;
 typedef signed   int   S32;
 typedef float          FP;
 typedef U32            OS_STK_TYPE;

/*
************************************************************
*                       SAVED AREA
************************************************************
*/
 #ifdef __IAR_SYSTEMS_ICC__
   extern __arm OS_STK_TYPE OSEnterCrit(void);             // here you need the "OS_AT91SAM7Sxx_c.c" file
   extern __arm void OSExitCrit(OS_STK_TYPE);
 #else
  #ifdef __KEIL__
   extern OS_STK_TYPE OSEnterCrit(void) __arm; 
   extern void OSExitCrit(OS_STK_TYPE) __arm;
  #else
   extern OS_STK_TYPE OSEnterCrit(void); 
   extern void OSExitCrit(OS_STK_TYPE);
  #endif
 #endif

 #define OS_ENTER_CRITICAL() OS_sr = OSEnterCrit()
 #define OS_EXIT_CRITICAL()  OSExitCrit(OS_sr)

 #define OS_TASK_SW()        OSCtxSw()

   /*
************************************************************
*                       CONSTANTS
************************************************************
*/
 #ifndef FALSE
  #define FALSE  0
 #endif
 #ifndef TRUE
  #define TRUE   1
 #endif
 #ifndef NULL
  #define NULL   ((void *)0)
 #endif

   /*
************************************************************
*                       REGISTERS
************************************************************
*/

 #include <SAM3S.H>

/************************** END ***************************/
#endif
