/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the �C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                       FE310_regs.h                       *
*             RISC-V FE310 Register definitions            *
*                                                          *
***********************************************************/
#ifndef _FE310_REGS_
 #define _FE310_REGS_

#include <sys/types.h>
#include <stdint.h>

#define __IO volatile

#if F_CPU % 8000000
#error "F_CPU: only multiples of 8Mhz supported (64 - 384Mhz please)"
#endif
#define PLL_F_VAL ((F_CPU/8000000)-1)



#define read_csr(reg) ({ unsigned long __tmp; \
  asm volatile ("csrr %0, " #reg : "=r"(__tmp)); \
  __tmp; })

#define write_csr(reg, val) ({ \
  if (__builtin_constant_p(val) && (unsigned long)(val) < 32) \
    asm volatile ("csrw " #reg ", %0" :: "i"(val)); \
  else \
    asm volatile ("csrw " #reg ", %0" :: "r"(val)); })

#define swap_csr(reg, val) ({ unsigned long __tmp; \
  if (__builtin_constant_p(val) && (unsigned long)(val) < 32) \
    asm volatile ("csrrw %0, " #reg ", %1" : "=r"(__tmp) : "i"(val)); \
  else \
    asm volatile ("csrrw %0, " #reg ", %1" : "=r"(__tmp) : "r"(val)); \
  __tmp; })

#define set_csr(reg, bit) ({ unsigned long __tmp; \
  if (__builtin_constant_p(bit) && (unsigned long)(bit) < 32) \
    asm volatile ("csrrs %0, " #reg ", %1" : "=r"(__tmp) : "i"(bit)); \
  else \
    asm volatile ("csrrs %0, " #reg ", %1" : "=r"(__tmp) : "r"(bit)); \
  __tmp; })

#define clear_csr(reg, bit) ({ unsigned long __tmp; \
  if (__builtin_constant_p(bit) && (unsigned long)(bit) < 32) \
    asm volatile ("csrrc %0, " #reg ", %1" : "=r"(__tmp) : "i"(bit)); \
  else \
    asm volatile ("csrrc %0, " #reg ", %1" : "=r"(__tmp) : "r"(bit)); \
  __tmp; })

#define IRQ_S_SOFT   1
#define IRQ_H_SOFT   2
#define IRQ_M_SOFT   3
#define IRQ_S_TIMER  5
#define IRQ_H_TIMER  6
#define IRQ_M_TIMER  7
#define IRQ_S_EXT    9
#define IRQ_H_EXT    10
#define IRQ_M_EXT    11
#define IRQ_COP      12
#define IRQ_HOST     13

#define MIP_SSIP            (1 << IRQ_S_SOFT)
#define MIP_HSIP            (1 << IRQ_H_SOFT)
#define MIP_MSIP            (1 << IRQ_M_SOFT)
#define MIP_STIP            (1 << IRQ_S_TIMER)
#define MIP_HTIP            (1 << IRQ_H_TIMER)
#define MIP_MTIP            (1 << IRQ_M_TIMER)
#define MIP_SEIP            (1 << IRQ_S_EXT)
#define MIP_HEIP            (1 << IRQ_H_EXT)
#define MIP_MEIP            (1 << IRQ_M_EXT)

#define MSTATUS_MIE         (1 << 3)
#define MSTATUS_MPIE        (1 << 7)
#define MSTATUS_MPP         (3 << 11)
/*
************************************************************
*                  exceptions & interrupts
************************************************************
*/
#define mcause_exc_iam                     0x00000000
#define mcause_exc_iaf                     0x00000001
#define mcause_exc_ii                      0x00000002
#define mcause_exc_break                   0x00000003
#define mcause_exc_lam                     0x00000004
#define mcause_exc_laf                     0x00000005
#define mcause_exc_sam                     0x00000006
#define mcause_exc_saf                     0x00000007
#define mcause_exc_ecfM                    0x0000000B
#define mcause_int_msi                     0x80000003
#define mcause_int_tsi                     0x80000007
#define mcause_int_mei                     0x8000000B     // see PLIC interrupts

#define PLIC_int_nr_WDT                    1
#define PLIC_int_nr_RTC                    2
#define PLIC_int_nr_UART0                  3
#define PLIC_int_nr_UART1                  4
#define PLIC_int_nr_QSPI0                  5
#define PLIC_int_nr_QSPI1                  6
#define PLIC_int_nr_QSPI2                  7
#define PLIC_int_nr_GPIO(x)                ( 8 + (x & 0x1F))
#define PLIC_int_mr_PWM0cmp(x)             (40 + (x & 0x03))
#define PLIC_int_nr_PWM1cmp(x)             (43 + (x & 0x03))
#define PLIC_int_nr_PWM2cmp(x)             (47 + (x & 0x03))

#define PLIC_int_pending_enable_bit(nr)    (1 << nr)

/*
************************************************************
*                           CLINT
************************************************************
*/
#define FE310_CLINT_REG_BASE    0x02000000UL

typedef struct {
    __IO uint32_t    msip;           // 02000000 - MSIP Registers
         uint32_t    Rsvda[4095];    // 02000004..02003FFF
    __IO uint64_t    mtimecmp;       // 02004000 - Timer compare register
         uint32_t    Rsvdb[8188];    // 02004008..0200BFF7
    __IO uint64_t    mtime;          // 0200BFF8 - Timer register
} CLINT_TypeDef;

#define CLINT                   ((CLINT_TypeDef *) FE310_CLINT_REG_BASE)

/*
************************************************************
*                           PLIC
************************************************************
*/
#define FE310_PLIC_REG_BASE     0x0C000000UL

typedef struct {
    __IO uint32_t    priority[52];   // 0C000000 - Priority Register
         uint32_t    Rsvda[972];     // 0C0000D0..0C000FFF
    __IO uint32_t    pending[2];     // 0C001000 - Interrupt Pending Register 
         uint32_t    Rsvdb[1022];    // 0C001008..0C001FFF
    __IO uint32_t    enable[2];      // 0C002000 - Interrupt Enable Register
         uint32_t    Rsvdc[522232];  // 0C002020..0C1FFFFF
    __IO uint32_t    threshold;      // 0C200000 - Priority Threshold Register
    __IO uint32_t    claim;          // 0C200004 - Claim/Complete Register
} PLIC_TypeDef;

#define PLIC                    ((PLIC_TypeDef *) FE310_PLIC_REG_BASE)

#define PLIC_priority(plic, int_nr, prio)   (plic->priority[int_nr] = (prio & 0x07))
#define PLIC_pending(plic, int_nr)   (plic->pending[int_nr / 32] & PLIC_int_pending_enable_bit(int_nr % 32))
#define PLIC_enable(plic, int_nr)    (plic->enable[int_nr / 32] |= PLIC_int_pending_enable_bit(int_nr % 32))
#define PLIC_disable(plic, int_nr)   (plic->enable[int_nr / 32] &= ~PLIC_int_pending_enable_bit(int_nr % 32))

/*
************************************************************
*                           AON
************************************************************
*/
#define FE310_AON_REG_BASE      0x10000000UL

/*
************************************************************
*                           PMU
************************************************************
*/
#define FE310_PMU_REG_BASE      (FE310_AON_REG_BASE + 0x0080)

typedef struct {
         uint32_t    backup[32];     // 080 - backup register
         uint32_t    wakeup[8];      // 100 - 32byte wakeup program memory
         uint32_t    sleed[8];       // 120 - 32byte sleep program memory
    __IO uint32_t    pmuie;          // 140 - interrupt enables
    __IO uint32_t    pmucause;       // 144 - wakeup cause
    __IO uint32_t    pmusleep;       // 148 - initiate sleep sequence
    __IO uint32_t    pmukey;         // 14C - key register
} PMU_TypeDef;

#define PMU                     ((PMU_TypeDef *) FE310_PMU_REG_BASE)

#define PMU_pmuie_rtc                (1 << 1)
#define PMU_pmuie_dwakeup            (1 << 2)
#define PMU_pmucause_wakeup_mask     (3 << 0)
#define PMU_pmucause_reset_mask      (3 << 8)

/*
************************************************************
*                           WDT
************************************************************
*/
#define FE310_WDT_REG_BASE      (FE310_AON_REG_BASE + 0x0000)

typedef struct {
    __IO uint32_t    wdogcfg;        // 00 - Configuration Register
         uint32_t    Rsvda;          // 04
    __IO uint32_t    wdogcount;      // 08 - Count Register
         uint32_t    Rsvdb;          // 0C
    __IO uint32_t    wdogs;          // 10 - ?
         uint32_t    Rsvdc;          // 14
    __IO uint32_t    wdogfeed;       // 18 - Feed Address
    __IO uint32_t    wdogkey;        // 1C - Key Register
    __IO uint32_t    wdogcmp;        // 20 - Compare Register
} WDT_TypeDef;

#define WDT                     ((WDT_TypeDef *) FE310_WDT_REG_BASE)

#define WDT_wdogcfg_wdogscale_mask   (7 <<  0)
#define WDT_wdogcfg_wdogrsten        (1 <<  8)
#define WDT_wdogcfg_wdogzerocmp      (1 <<  9)
#define WDT_wdogcfg_wdogenalways     (1 << 12)
#define WDT_wdogcfg_wdogencoreawake  (1 << 13)
#define WDT_wdogcfg_wdogcmpip        (1 << 28)
#define WDT_wdogcmp_wdogcmp_mask     (0xFF << 0)


/*
************************************************************
*                           RTC
************************************************************
*/
#define FE310_RTC_REG_BASE      (FE310_AON_REG_BASE + 0x0040)

typedef struct {
    __IO uint32_t    rtccfg;         // 00 - Configuration Register
         uint32_t    Rsvda;          // 04
    __IO uint32_t    rtclo;          // 08 - Counter Register Low
    __IO uint32_t    rtchi;          // 0C - Counter Register High
    __IO uint32_t    rtcs;           // 10
         uint32_t    Rsvdb[3];       // 14..1C
    __IO uint32_t    rtccmp;         // 20 - Counter Compare Register
         uint32_t    Rsvdc[19];      // 24..6C    
    __IO uint32_t    lfrosccfg;      // 70 - LF Ring Oscillator Configuration Register
} RTC_TypeDef;

#define RTC                     ((RTC_TypeDef *) FE310_RTC_REG_BASE)

#define RTC_rtccfg_rtcscale_mask     (0x0F << 0)
#define RTC_rtccfg_rtcenalways       (1 << 12)
#define RTC_rtccfg_rtcmpip           (1 << 28)
#define RTC_rtchi_rtchi_mask         (0xFF << 0)
#define RTC_lfrosccfg_en             (1 << 30)
/*
typedef struct {
    WDT  wdg;
    RTC  rtc;
    PMU  pmu;
} AON_TypeDef;

#define AON                     AON_TypeDef
*/
/*
************************************************************
*                           PRCI
************************************************************
*/
#define FE310_PRCI_REG_BASE     0x10008000UL

typedef struct {
    __IO uint32_t    hfrosccfg;      // 00 - HF Ring Oscillator Configuration Register
    __IO uint32_t    hfxosccfg;      // 04 - HF Crystal Oscillator Configuration Register
    __IO uint32_t    pllcfg;         // 08 - PLL Configuration Register
    __IO uint32_t    plloutdiv;      // 0C - PLL Output Divider Register
} PRCI_TypeDef;

#define PRCI                    ((PRCI_TypeDef *) FE310_PRCI_REG_BASE)

#define PRCI_hfrosccfg_hfroscdiv_mask      0x000003FF
#define PRCI_hfrosccfg_hfroscdiv_offs      0
#define PRCI_hfrosccfg_hfroscdiv_s(x)      ((x << PRCI_hfrosccfg_hfroscdiv_offs) & PRCI_hfrosccfg_hfroscdiv_mask)
#define PRCI_hfrosccfg_hfroscdiv_g(x)      ((x & PRCI_hfrosccfg_hfroscdiv_mask) >> PRCI_hfrosccfg_hfroscdiv_offs)
#define PRCI_hfrosccfg_hfrosctrim_mask     0x001F0000
#define PRCI_hfrosccfg_hfrosctrim_offs     16
#define PRCI_hfrosccfg_hfrosctrim_s(x)     ((x << PRCI_hfrosccfg_hfrosctrim_offs) & PRCI_hfrosccfg_hfrosctrim_mask)
#define PRCI_hfrosccfg_hfrosctrim_g(x)     ((x & PRCI_hfrosccfg_hfrosctrim_mask) >> PRCI_hfrosccfg_hfrosctrim_offs)
#define PRCI_hfrosccfg_hfroscen            (1 << 30)
#define PRCI_hfrosccfg_hfroscrdy           (1 << 31)
#define PRCI_hfxosccfg_hfxoscen            (1 << 30)
#define PRCI_hfxosccfg_hfxoscrdy           (1 << 31)
#define PRCI_pllcfg_pllr_mask              (0x07UL <<  0)
#define PRCI_pllcfg_pllr_offs              0
#define PRCI_pllcfg_pllr_s(x)              ((x << PRCI_pllcfg_pllr_offs) & PRCI_pllcfg_pllr_mask)
#define PRCI_pllcfg_pllf_mask              (0x3FUL <<  4)
#define PRCI_pllcfg_pllf_offs              4
#define PRCI_pllcfg_pllf_s(x)              ((x << PRCI_pllcfg_pllf_offs) & PRCI_pllcfg_pllf_mask)
#define PRCI_pllcfg_pllq_mask              (0x03UL << 10)
#define PRCI_pllcfg_pllq_offs              10
#define PRCI_pllcfg_pllq_s(x)              ((x << PRCI_pllcfg_pllq_offs) & PRCI_pllcfg_pllq_mask)
#define PRCI_pllcfg_pllsel                 (1 << 16)
#define PRCI_pllcfg_pllrefsel              (1 << 17)
#define PRCI_pllcfg_pllbypass              (1 << 18)
#define PRCI_pllcfg_plllock                (1 << 31)
#define PRCI_plloutdiv_plloutdiv_mask      (0x3FUL << 0)
#define PRCI_plloutdiv_plloutdiv_offs      0
#define PRCI_plloutdiv_plloutdiv_s(x)      ((x << PRCI_plloutdiv_plloutdiv_offs) & PRCI_plloutdiv_plloutdiv_mask)
#define PRCI_plloutdiv_plloutdivby1        (1 <<  8)

/*
************************************************************
*                           OTP
************************************************************
*/
#define FE310_OTP_REG_BASE      0x10010000UL

typedef struct {
    __IO uint32_t    lock;           // 00 - Programmed-I/O lock register
    __IO uint32_t    ck;             // 04 - device clock signal
    __IO uint32_t    oe;             // 08 - device output-enable signal
    __IO uint32_t    sel;            // 0C - device chip-select signal
    __IO uint32_t    we;             // 10 - device write-enable signal
    __IO uint32_t    mr;             // 14 - device mode register
    __IO uint32_t    mrr;            // 18 - read-voltage regulator control
    __IO uint32_t    mpp;            // 1C - write-voltage charge pump control
    __IO uint32_t    vrren;          // 20 - read-voltage enable
    __IO uint32_t    vppen;          // 24 - write-voltage enable
    __IO uint32_t    a;              // 28 - device address
    __IO uint32_t    d;              // 2C - device data input
    __IO uint32_t    q;              // 30 - device data output
    __IO uint32_t    rsctrl;         // 34 - read sequencer control
} OTP_TypeDef;

#define OTP                     ((OTP_TypeDef *) FE310_OTP_REG_BASE)

/*
************************************************************
*                           GPIO
************************************************************
*/
#define FE310_GPIO0_REG_BASE    0x10012000UL

typedef struct {
    __IO uint32_t    value;          // 00 - pin value
    __IO uint32_t    input_en;       // 04 - async pin input enable
    __IO uint32_t    output_en;      // 08 - async pin output enable
    __IO uint32_t    port;           // 0C - output port value
    __IO uint32_t    pue;            // 10 - async internal pull-up enable
    __IO uint32_t    ds;             // 14 - pin drive strength
    __IO uint32_t    rise_ie;        // 18 - rise interrupt enable
    __IO uint32_t    rise_ip;        // 1C - rise interrupt pending
    __IO uint32_t    fall_ie;        // 20 - fall interrupt enable
    __IO uint32_t    fall_ip;        // 24 - fall interrupt pending
    __IO uint32_t    high_ie;        // 28 - high interrupt enable
    __IO uint32_t    high_ip;        // 2C - high interrupt pending
    __IO uint32_t    low_ie;         // 30 - low interrupt enable
    __IO uint32_t    low_ip;         // 34 - low interrupt pending
    __IO uint32_t    iof_en;         // 38 - async HW I/O Function enable
    __IO uint32_t    iof_sel;        // 3C - HW I/O Function select
    __IO uint32_t    out_xor;        // 40 - Output XOR (invert)
} GPIO_TypeDef;

#define GPIO0                   ((GPIO_TypeDef *) FE310_GPIO0_REG_BASE)

#define GPIO_pin(nr)                 (1 << nr)

/*
************************************************************
*                           SPI
************************************************************
*/
#define FE310_SPI0_REG_BASE     0x10014000UL
#define FE310_SPI1_REG_BASE     0x10024000UL
#define FE310_SPI2_REG_BASE     0x10034000UL

typedef struct {
    __IO uint32_t    sckdiv;         // 00 - Serial clock divisor
    __IO uint32_t    sckmode;        // 04 - Serial clock mode
         uint32_t    Rsvda[2];       // 08..0C
    __IO uint32_t    csid;           // 10 - Chip select ID
    __IO uint32_t    csdef;          // 14 - Chip select default
    __IO uint32_t    csmode;         // 18 - Chip select mode
         uint32_t    Rsvdb[3];       // 1C..24
    __IO uint32_t    delay0;         // 28 - Delay control 0
    __IO uint32_t    delay1;         // 2C - Delay control 1
         uint32_t    Rsvdc[4];       // 30..3C
    __IO uint32_t    fmt;            // 40 - Frame format
         uint32_t    Rsvdd;          // 44
    __IO uint32_t    txdata;         // 48 - Tx FIFO data
    __IO uint32_t    rxdata;         // 4C - Rx FIFO data
    __IO uint32_t    txmark;         // 50 - Tx FIFO watermark
    __IO uint32_t    rxmark;         // 54 - Rx FIFO watermark
         uint32_t    Rsvde[2];       // 58..5C
    __IO uint32_t    fctrl;          // 60 - async SPI flash interface control
    __IO uint32_t    ffmt;           // 64 - async SPI flash instruction format
         uint32_t    Rsvdf[2];       // 68..6C
    __IO uint32_t    ie;             // 70 - SPI interrupt enable
    __IO uint32_t    ip;             // 74 - SPI interrupt pending
} SPI_TypeDef;

#define SPI0                    ((SPI_TypeDef *) FE310_SPI0_REG_BASE)
#define SPI1                    ((SPI_TypeDef *) FE310_SPI1_REG_BASE)
#define SPI2                    ((SPI_TypeDef *) FE310_SPI2_REG_BASE)

#define SPI_sckdiv_scale_mask        (0x0FFF << 0)
#define SPI_sckmode_pha              (1 << 0)
#define SPI_sckmode_pol              (1 << 1)
#define SPI_csmode_mask              (3 << 0)
#define SPI_csmode_auto              (0 << 0)
#define SPI_csmode_hold              (2 << 0)
#define SPI_csmode_off               (3 << 0)
#define SPI_delay0_cssck_mask        (0x0F << 0)
#define SPI_delay0_cssck_offs        0
#define SPI_delay0_cssck_s(x)        ((x << SPI_delay0_cssck_offs) & SPI_delay0_cssck_mask)
#define SPI_delay0_sckcs_mask        (0x0F << 16)
#define SPI_delay0_sckcs_offs        16
#define SPI_delay0_sckcs_s(x)        ((x << SPI_delay0_sckcs_offs) & SPI_delay0_sckcs_mask)
#define SPI_delay1_intercs_mask      (0x0F << 0)
#define SPI_delay1_intercs_offs      0
#define SPI_delay1_intercs_s(x)      ((x << SPI_delay1_intercs_offs) & SPI_delay1_intercs_mask)
#define SPI_delay1_interxfr_mask     (0x0F << 16)
#define SPI_delay1_interxfr_offs     16
#define SPI_delay1_interxfr_s(x)     ((x << SPI_delay1_interxfr_offs) & SPI_delay1_interxfr_mask)
#define SPI_fmt_proto_mask           (3 << 0)
#define SPI_fmt_proto_offs           0
#define SPI_fmt_proto_single         0
#define SPI_fmt_proto_dual           1
#define SPI_fmt_proto_quad           2
#define SPI_fmt_endian               (1 << 2)
#define SPI_fmt_endian_msb           (0 << 2)
#define SPI_fmt_endian_lsb           (1 << 2)
#define SPI_fmt_dir                  (1 << 3)
#define SPI_fmt_len_mask             (0x0F << 16)
#define SPI_fmt_len_offs             16
#define SPI_fmt_len_s(x)             ((x << SPI_fmt_len_offs) & SPI_fmt_len_mask)
#define SPI_txdata_mask              (0xFF << 0)
#define SPI_txdata_full              (1 << 31)
#define SPI_rxdata_mask              (0xFF << 0)
#define SPI_rxdata_empty             (1 << 31)
#define SPI_txmark_mask              (7 << 0)
#define SPI_rxmark_mask              (7 << 0)
#define SPI_fctrl_en                 (1 << 0)
#define SPI_ffmt_cmd_en              (1 << 0)
#define SPI_ffmt_addr_len_mask       (7 << 1)
#define SPI_ffmt_addr_len_offs       1
#define SPI_ffmt_pad_cnt_mask        (0xF << 4)
#define SPI_ffmt_pad_cnt_offs        4
#define SPI_ffmt_cmd_proto_mask      (3 << 8)
#define SPI_ffmt_cmd_proto_offs      8
#define SPI_ffmt_addr_proto_mask     (3 << 10)
#define SPI_ffmt_addr_proto_offs     10
#define SPI_ffmt_data_proto_mask     (3 << 12)
#define SPI_ffmt_data_proto_offs     12
#define SPI_ffmt_cmd_code_mask       (0xFF << 16)
#define SPI_ffmt_cmd_code_offs       16
#define SPI_ffmt_pad_code_mask       (0xFF << 24)
#define SPI_ffmt_pad_code_offs       24
#define SPI_ie_txwm                  (1 << 0)
#define SPI_ie_rxwm                  (1 << 1)
#define SPI_ip_txwm                  (1 << 0)
#define SPI_ip_rxwm                  (1 << 1)

/*
************************************************************
*                           UART
************************************************************
*/
#define FE310_UART0_REG_BASE    0x10013000UL
#define FE310_UART1_REG_BASE    0x10023000UL

typedef struct {
    __IO uint32_t    txdata;         // 00 - Transmit data register
    __IO uint32_t    rxdata;         // 04 - Receive data register
    __IO uint32_t    txctrl;         // 08 - Transmit control register
    __IO uint32_t    rxctrl;         // 0C - Receive control register
    __IO uint32_t    ie;             // 10 - UART interrupt enable
    __IO uint32_t    ip;             // 14 - UART Interrupt pending
    __IO uint32_t    div;            // 18 - Baud rate divisor
} UART_TypeDef;

#define UART0                   ((UART_TypeDef *) FE310_UART0_REG_BASE)
#define UART1                   ((UART_TypeDef *) FE310_UART1_REG_BASE)

#define UART_txdata_mask             (0xFF << 0)
#define UART_txdata_full             (1 << 31)
#define UART_rxdata_mask             (0xFF << 0)
#define UART_rxdata_empty            (1 << 31)
#define UART_txctrl_txen             (1 << 0)
#define UART_txctrl_nstop            (1 << 1)
#define UART_txctrl_txcnt_mask       (7 << 16)
#define UART_txctrl_txcnt_offs       16
#define UART_rxctrl_rxen             (1 << 0)
#define UART_rxctrl_rxcnt_mask       (7 << 16)
#define UART_rxctrl_rxcnt_offs       16
#define UART_ie_txwm                 (1 << 0)
#define UART_ie_rxwm                 (1 << 1)
#define UART_ip_txwm                 (1 << 0)
#define UART_ip_rxwm                 (1 << 1)
#define UART_div_div_mask            (0xFFFF << 0)

/*
************************************************************
*                           PWM
************************************************************
*/
#define FE310_PWM0_REG_BASE     0x10015000UL
#define FE310_PWM1_REG_BASE     0x10025000UL
#define FE310_PWM2_REG_BASE     0x10035000UL

typedef struct {
    __IO uint32_t    pwmcfg;         // 00 - Configuration Register
         uint32_t    Rsvda;          // 04
    __IO uint32_t    pwmcount;       // 08 - Count Register
         uint32_t    Rsvdb;          // 0C
    __IO uint32_t    pwms;           // 10 - current value ?
         uint32_t    RRsvdc[3];      // 14..1C
    __IO uint32_t    pwmcmp[4];      // 20..2C - Compare Registers 0..4
} PWM_TypeDef;

#define PWM0                    ((PWM_TypeDef *) FE310_PWM0_REG_BASE)
#define PWM1                    ((PWM_TypeDef *) FE310_PWM1_REG_BASE)
#define PWM2                    ((PWM_TypeDef *) FE310_PWM2_REG_BASE)

#define PWM_pwmcfg_pwmscale          (0xF << 0)
#define PWM_pwmcfg_pwmsticky         (1 <<  8)
#define PWM_pwmcfg_pwmzerocmp        (1 <<  9)
#define PWM_pwmcfg_pwmdeglitch       (1 << 10)
#define PWM_pwmcfg_pwmenalways       (1 << 12)
#define PWM_pwmcfg_pwmenoneshot      (1 << 13)
#define PWM_pwmcfg_pwmcmp0center     (1 << 16)
#define PWM_pwmcfg_pwmcmp1center     (1 << 17)
#define PWM_pwmcfg_pwmcmp2center     (1 << 18)
#define PWM_pwmcfg_pwmcmp3center     (1 << 19)
#define PWM_pwmcfg_pwmcmp0gang       (1 << 24)
#define PWM_pwmcfg_pwmcmp1gang       (1 << 25)
#define PWM_pwmcfg_pwmcmp2gang       (1 << 26)
#define PWM_pwmcfg_pwmcmp3gang       (1 << 27)
#define PWM_pwmcfg_pwmcmp0ip         (1 << 28)
#define PWM_pwmcfg_pwmcmp1ip         (1 << 29)
#define PWM_pwmcfg_pwmcmp2ip         (1 << 30)
#define PWM_pwmcfg_pwmcmp3ip         (1 << 31)
#define PWM_pwmcmp_pwmcmp_mask       (0xFFFF << 0)

/************************** END ***************************/
#endif
