/***********************************************************
*                       pC/OS 1.88c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                         OS_cfg.h                         *
*                   pC/OS CONFIGURATION                    *
***********************************************************/

#ifndef _OS_cfg_
 #define _OS_cfg_

/*
************************************************************
*                 COMPONENTS CONFIGURATION                 *
************************************************************
*/
 #define OS_SYSTEM_TICKS_PER_SEC   10     /* System ticks per secound                                */
 #define OS_TIMER_TICKS_PER_SEC    100    /* Timer ticks per secound                                 */

 #define OS_TASK_EXT_EN            1      /* Include code for extended TASKS services           1    */
 #define OS_TASK_DESTROY_EN        1      /* Include code for delete pending/waiting TASKS     (1)   */
 #define OS_SEM_EN                 1      /* Include code for SEMAPHORES                             */
 #define OS_SEM_EXT_EN             1      /* Include code for extended SEMAPHORES services           */
 #define OS_MUX_EN                 1      /* Include code for MUTEXES                                */
 #define OS_MBOX_EN                1      /* Include code for MAILBOXES                              */
 #define OS_Q_EN                   1      /* Include code for QUEUES                                 */
 #define OS_P_EN                   1      /* Include code for PIPES                                  */
 #define OS_EVG_EN                 1      /* Include code for EVENTGROUPS                            */
 #define OS_TMR_EN                 1      /* Include code for TIMERS                                 */
 #define OS_MEM_EN                 1      /* Include code for MEMORY                                 */
 #define OS_HIS_EN                 1      /* Include code for HISTORY                                */

 #define OS_STK_CHECK_EN           1      /* check end-of-stack of old task during context switch    */
 #define OS_STK_CHECK_FILL         1      /* fill stack with 0xEF pattern to get the deep of use     */

 #define OS_MEM_CHECK_EN           1      /* check memory-links on alloc & free (only using MEMORY)  */

/*
************************************************************
*                    USER CONFIGURATION                    *
************************************************************
*/
 #define OS_MAX_TASKS              16     /* max created tasks in hole system  -->  max 64 !         */
 #define OS_MIN_PRIO               16     /* lowest possible prio              -->  max 64 !         */
 #define OS_IDLE_STK_SIZE          128    /* idle stack size in OS_STK_TYPE ...                      */
                                          /* ... with (OS_MIN_PRIO - 1) as prio for idle task        */
 #define OS_TMR_PRIO   (OS_MIN_PRIO - 2)  /* timer task prio                                         */
 #define OS_TMR_STK_SIZE           128    /* timer stack size in OS_STK_TYPE                         */
 #define OS_MAX_HISTORY            100    /* history entries                                         */
 #define OS_STK_RESERVE            4      /* space between real end-of-stack and check-point ...     */
                                          /* ... in OS_STK_TYPE                                      */

/*
************************************************************
*                    CPU CONFIGURATION                     *
************************************************************
*/
// #include "8051/OS_8051.h"                /* 8051                                                    */
// #include "Rabbit/OS_Rabbit.h"            /* Rabbit 2000/3000                                        */
// #include "Z180/OS_Z180.h"                /* Zilog Z18x                                              */
// #include "MSP430/OS_MSP430.h"            /* TI MSP430Fxxx                                           */
// #include "PhilipsXA/OS_PhilipsXA.h"      /* Philips XA                                              */
// #include "186/OS_186.h"                  /* Intel/AMD 80188/80186                                   */
// #include "ATMega32/OS_ATMega32.h"        /* Atmel AVR ATMega32                                      */
// #include "ATMega128/OS_ATMega128.h"      /* Atmel AVR ATMega128                                     */
// #include "AVR32/OS_AVR32.h"              /* Atmel AVR32                                             */
// #include "AT91SAM7xxx/OS_AT91SAM7xxx.h"  /* Atmel ARM7TDMI  AT91SAM7... (S/X/XC/SE/L)               */
// #include "AT91SAM3xxx/OS_AT91SAM3xxx.h"  /* Atmel Cortex-M3 AT91SAM3... (S/U/..)                    */
 #include "SAMD21/OS_SAMD21.h"            /* Atmel/Microchip Cortex-M0  SAMD21...                    */
// #include "SAMD51/OS_SAMD51.h"            /* Atmel/Microchip Cortex-M4F SAMD51...                    */
// #include "SAMS70/OS_SAMS70.h"            /* Atmel/Microchip Cortex-M7 SAMS70...                     */
// #include "SAML10/OS_SAML10.h"            /* Atmel/Microchip Cortex-M23 SAMDL10 w/o-TEE              */
// #include "SAML11/OS_SAML11.h"            /* Atmel/Microchip Cortex-M23 SAMDL11 with TEE             */
// #include "EFM32/OS_EFM32.h"              /* SiliconLabs Cortex-M0 EFM32 Happy Gecko                 */
// #include "NUC123/OS_NUC123.h"            /* Nuvoton Cortex-M0 NUC123...                             */
// #include "M2351/OS_M2351.h"              /* Nuvoton Cortex-M23 M2351 with TEE                       */
// #include "LM3S811/OS_LM3S811.h"          /* Luminary Cortex-M3 LM3S811                              */
// #include "LM3S9B92/OS_LM3S9B92.h"        /* TI Luminary Cortex-M3 LM3S9B92                          */
// #include "TMS570/OS_TMS570.h"            /* TI Cortex-R4 TMS5700432 w/o FPU                         */
// #include "LPC214x/OS_LPC214x.h"          /* NXP ARM7TDMI  LPC214x                                   */
// #include "LPC236x/OS_LPC236x.h"          /* NXP ARM7TDMI  LPC236x                                   */
// #include "LPC11xx/OS_LPC11xx.h"          /* NXP Cortex-M0 LPC11xx                                   */
// #include "LPC13xx/OS_LPC13xx.h"          /* NXP Cortex-M3 LPC13xx                                   */
// #include "LPC17xx/OS_LPC17xx.h"          /* NXP Cortex-M3 LPC17xx                                   */
// #include "MKL25Z128/OS_MKL25Z128.h"      /* Freescale Cortex-M0 MKL25Z128xxx4                       */
// #include "MKL26Z64/OS_MKL26Z64.h"        /* Freescale Cortex-M0 MKL26Z64xxx4                        */
// #include "MK20DX256/OS_MK20DX256.h"      /* Freescale Cortex-M4 MK20DX256xxx7                       */
// #include "MK60DN512/OS_MK60DN512.h"      /* Freescale Cortex-M4 MK60DN512xxx10                      */
// #include "MK66FX1M0/OS_MK66FX1M0.h"      /* Freescale Cortex-M4 MK66FX1M0xxx18                      */
// #include "nRF52832/OS_nRF52832.h"        /* Nordic Semiconductor Cortex-M4 nRF52832                 */
// #include "STR711/OS_STR711.h"            /* ST ARM7TDMI  STR711                                     */
// #include "STM32F0xx/OS_STM32F0xx.h"      /* STM Cortex-M0 STM32F0xx                                 */
// #include "STM32F1xx/OS_STM32F1xx.h"      /* STM Cortex-M3 STM32F1xx                                 */
// #include "STM32F4xx/OS_STM32F4xx.h"      /* STM Cortex-M4 STM32F4xx                                 */
// #include "STM32F7xx/OS_STM32F7xx.h"      /* STM Cortex-M7 STM32F7xx                                 */
// #include "STM32L4xx/OS_STM32L4xx.h"      /* STM Cortex-M4 STM32L4xx                                 */
// #include "XMC1100/OS_XMC1100.h"          /* Infineon Cortex-M0 XMC1100                              */
// #include "PIC32MX/OS_PIC32.h"            /* Microchip MIPS32-4k PIC32MX                             */
// #include "PIC32MZ/OS_PIC32.h"            /* Microchip MIPS32 M-Class PIC32MZ                        */
// #include "CortexM23/OS_CortexM23.h"      /* ARMv8M BaseLine Cortex-M23                              */
// #include "CortexM33/OS_CortexM33.h"      /* ARMv8M MainLine Cortex-M33                              */
// #include "FE310_RISCV/OS_FE310.h"        /* SiFive RISC-V Freedom E310                              */
// #include "ARC_EM/OS_ARC_EM.h"            /* Synopsis ARC-EM generic                                 */
// #include "R32C/OS_R32C.h"                /* Renesas R32C111                                         */
// #include "RX210/OS_RX210.h"              /* Renesas RX210                                           */
// #include "RX62N/OS_RX62N.h"              /* Renesas RX62N                                           */
// #include "V850/OS_V850.h"                /* Renesas V850                                            */
// #include "RL78/OS_RL78.h"                /* Renesas RL78                                            */
// #include "GD32E230/OS_GD32E230.h"        /* GigaDevices Cortex-M23 GD32E230 w/o-TEE                 */
// #include "MPC5744/OS_MPC5744.h"          /* NXP/Freescale PowerPC e200z4                            */

/************************* END ****************************/
#endif

