/***********************************************************
*                       pC/OS 1.88c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                         pC_OS.h                          *
*                   SYSTEM DECLARATIONS                    *
***********************************************************/
#ifndef _pC_OS_
 #define _pC_OS_

#include "../inc/OS_cfg.h"

/*
************************************************************
*                   SYSTEM DECLARATIONS                    *
************************************************************
*/
#if (OS_MAX_TASKS > 64)         /* check max tasks        */
 #error "only max 64 tasks supported !"
#endif
#if (OS_MIN_PRIO > 64)          /* check lowest priority  */
 #error "only max 64 prios supported !"
#endif
#if (OS_MIN_PRIO < OS_MAX_TASKS)
 #error "smaller prios than tasks !"
#endif                          /* IDLE task priority     */
#define OS_LO_PRIO (OS_MIN_PRIO - 1)
#if  OS_TMR_EN
 #if (OS_TMR_PRIO > (OS_MIN_PRIO - 2))
  #error "timer prio too low !" /* TIMER task priority    */ 
 #endif
#endif
#if (OS_MIN_PRIO % 8)           /* TABLE_DEEP of IPCs     */
 #define OS_PRIO_TBL (OS_MIN_PRIO / 8 + 1)
#else
 #define OS_PRIO_TBL (OS_MIN_PRIO / 8)
#endif

#define OS_PRIO_SELF      0xFF /* Indicate SELF prio      */
#define OS_ID_SELF        0xFF /* Indicate SELF id        */

                               /* TASK STATUS             */
#define OS_STAT_RDY       0x00 /* Ready to run            */
#define OS_STAT_MBOX      0x01 /* Pending on mailbox      */
#define OS_STAT_SEM       0x02 /* Pending on semaphore    */
#define OS_STAT_Q         0x04 /* Pending on queue        */
#define OS_STAT_P         0x08 /* Pending on pipe         */
#define OS_STAT_EVG       0x10 /* Pending on eventgroup   */
#define OS_STAT_MUX       0x20 /* Pending on mutex        */
#define OS_STAT_POST      0x40 /* Suspended on post       */
#define OS_STAT_SUSP      0x80 /* Suspended               */

                               /* PENDING MODE            */
#define OS_SUSPEND           0 /* Suspending              */
#define OS_NO_SUSP      0xFFFF /* no suspending           */

                               /* TIMER MODE              */
#define OS_TMR_ENABLE     0x80 /* enabled                 */
#define OS_TMR_MODE_M     0x03 /* mask for following bits */
#define OS_TMR_CLR        0x00 /* autocleared (run once)  */
#define OS_TMR_RONCE      0x01 /* run once (not cleared)  */
#define OS_TMR_CYCL       0x02 /* cyclic (autoreload)     */

                               /* EVENTGROUP MODE         */
#define OS_EVG_OR         0x01 /* OR-mode for eventgroup  */
#define OS_EVG_AND        0x02 /* AND-mode -"-            */
#define OS_EVG_CLR        0x02 /* CLR-mode -"-            */
#define OS_EVG_CONSUME    0x10 /* CONSUME events          */
#define OS_EVG_OR_C       (OS_EVG_OR | OS_EVG_CONSUME)
#define OS_EVG_AND_C      (OS_EVG_AND | OS_EVG_CONSUME)

/*
************************************************************
*                      ERROR CODES                         *
************************************************************
*/
#define OS_SUCCESS           0
#define OS_NO_ERR            0
#define OS_PARAM_ERR         1
#define OS_TIMEOUT          10

#define OS_MBOX_FULL        20
#define OS_MBOX_NODATA      21

#define OS_Q_FULL           30
#define OS_Q_NODATA         31
#define OS_Q_CLEAR          32

#define OS_PRIO_EXIST       40
#define OS_TASK_NOT_EXIST   41

#define OS_SEM_ERR          50
#define OS_SEM_NODATA       51
#define OS_SEM_OVF          52

#define OS_MUX_ERR          55
#define OS_MUX_NOACC        56
#define OS_MUX_USED         57

#define OS_P_FULL           60
#define OS_P_NODATA         61
#define OS_P_CLEAR          62
#define OS_P_LEN_ERR        63

#define OS_MEM_ERR          70
#define OS_MEM_OVF          71

#define OS_EVG_ERR          80
#define OS_EVG_NOE          81

#define OS_HIS_END          90

#define OS_SUSPEND_IDLE    100
#define OS_PRIO_INVALID    101
#define OS_TIME_NOT_DLY    102

#define OS_TASK_SUSP_PRIO  103
#define OS_TASK_NOT_SUSP   104
#define OS_TASK_NOT_RDY    105

#define OS_TMR_NO_TIME     106
#define OS_TMR_NOT_EXIST   107
#define OS_TMR_EXIST       108

#ifdef OS_PACK_PRAGMA
 #pragma pack(1)
#endif
/*
************************************************************
*            TASK CONTROL BLOCK DATA STRUCTURE             *
************************************************************
*/
OS_PACKED typedef struct os_tcb {
    void   OS_FAR *OSTCBStkPtr;
#if OS_STK_CHECK_EN
    void   OS_FAR *OSTCBStkEnd;
#endif
    U08            OSTCBStat;
    U08            OSTCBPrio;
    U16            OSTCBDly;
    struct os_tcb *OSTCBNext;
    struct os_tcb *OSTCBPrev;
#if OS_TASK_DESTROY_EN
    void   OS_FAR *OSTCBipc;
 #if OS_MUX_EN
    struct os_mux *OSTCBmux;
 #endif
#endif
    OS_STK_TYPE    OSTCBId;
} OS_PACKED_ATTR OS_TCB;

/*
************************************************************
*                 SEMAPHORE DATA STRUCTURE                 *
************************************************************
*/
OS_PACKED typedef struct os_sem {
    U16  OSSemCnt;
    U08  OSSemGrp;
    U08  OSSemTbl[OS_PRIO_TBL];
} OS_PACKED_ATTR OS_SEM;

/*
************************************************************
*                   MUTEX DATA STRUCTURE                   *
************************************************************
*/
OS_PACKED typedef struct os_mux {
    U08  OSMuxPrio;
    U08  OSMuxOrg;
    U08  OSMuxGrp;
    U08  OSMuxTbl[OS_PRIO_TBL];
} OS_PACKED_ATTR OS_MUX;

/*
************************************************************
*                  MAILBOX DATA STRUCTURE                  *
************************************************************
*/
OS_PACKED typedef struct os_mbox {
    void   *OSMboxMsg;
    BOOLEAN OSMboxValid;
    U08     OSMboxFGrp;
    U08     OSMboxFTbl[OS_PRIO_TBL];
    U08     OSMboxBGrp;
    U08     OSMboxBTbl[OS_PRIO_TBL];
} OS_PACKED_ATTR OS_MBOX;

/*
************************************************************
*                   QUEUE DATA STRUCTURE                   *
************************************************************
*/
OS_PACKED typedef struct os_q {
    U08 OS_HUGE *OSQData;
    U16          OSQIn;
    U16          OSQOut;
    U16          OSQSize;
    U16          OSQEntries;
    U08          OSQFGrp;
    U08          OSQFTbl[OS_PRIO_TBL];
    U08          OSQBGrp;
    U08          OSQBTbl[OS_PRIO_TBL];
} OS_PACKED_ATTR OS_Q;

/*
************************************************************
*                    PIPE DATA STRUCTURE                   *
************************************************************
*/
OS_PACKED typedef struct os_p {
    U08 OS_HUGE *OSPData;
    U08          OSPIn;
    U08          OSPOut;
    U16          OSPSize;
    U08          OSPDeep;
    U08          OSPEntries;
    U08 OS_HUGE *OSPData1;
    U16          OSPLenght1;
    U08          OSPFGrp;
    U08          OSPFTbl[OS_PRIO_TBL];
    U08          OSPBGrp;
    U08          OSPBTbl[OS_PRIO_TBL];
} OS_PACKED_ATTR OS_P;

/*
************************************************************
*                    TIMER DATA STRUCTURE                  *
************************************************************
*/
OS_PACKED typedef struct os_t {
    struct os_t OS_HUGE *OSTNext;
    struct os_t OS_HUGE *OSTPrev;
    void               (*OSTFct)(void *);
    void                *OSTArg;
    U32                  OSTValue;
    U32                  OSTCntEnd;
    U08                  OSTMode;
} OS_PACKED_ATTR OS_TMR;

/*
************************************************************
*                   MEMORY DATA STRUCTURE                  *
************************************************************
*/
OS_PACKED typedef struct os_m {
    struct os_m OS_HUGE *OSMNext;
    struct os_m OS_HUGE *OSMPrev;
    U32                  OSMLen;
#if OS_TASK_DESTROY_EN
    OS_STK_TYPE          OSMId;
#endif
} OS_PACKED_ATTR OS_MEM;

/*
************************************************************
*                EVENTGOUP SUPENDED STRUCTURE              *
************************************************************
*/
OS_PACKED typedef struct os_evg_susp {
    struct os_evg_susp OS_HUGE *OSsuspNext;
    U32                        *OSreqEvents;
    U08                         OSreqMode;
    U08                         OSsuspPrio;
} OS_PACKED_ATTR OS_EVG_SUSP;

/*
************************************************************
*                EVENTGOUP DATA STRUCTURE                  *
************************************************************
*/
OS_PACKED typedef struct os_evg {
    U32                  OSEvents;
    OS_EVG_SUSP OS_HUGE *OSEvgSuspList;
    U08                  OSEvgGrp;
    U08                  OSEvgTbl[OS_PRIO_TBL];
} OS_PACKED_ATTR OS_EVG;

/*
************************************************************
*                 HISTORY DATA STRUCTURE                   *
************************************************************
*/
OS_PACKED typedef struct os_history {
    U32    OSHPara1;
    U32    OSHPara2;
    U08    OSHPrio;
    U32    OSHTime;
} OS_PACKED_ATTR OS_H;

#ifdef OS_PACK_PRAGMA
 #pragma pack()
#endif

/*
************************************************************
*             COMPATIBILITY FOR UCOS-I PORTS               *
************************************************************
*/
#define  OS_IntEnter OSIntEnter
#define  OS_IntExit  OSIntExit

/*
************************************************************
*                pC/OS FUNCTION PROTOTYPES                 *
************************************************************
*/
void        OS_Init(void) KC51_REENTRANT;
void        OS_Start(void) KC51_REENTRANT;
U08 OS_FAR *OS_GetRev(void) KC51_REENTRANT;
#ifdef _OS_8051_
 U08        OS_TaskCreate(void (OS_FAR *ttask)(void *dptr), void *tdata, void *pstk, U16 stksize, U08 prio) KC51_REENTRANT;
#else
 #ifdef _OS_ATMega32_
  U08       OS_TaskCreate(void (OS_FAR *ttask)(void *dptr), void *tdata, void *pstk, U16 stksize, U08 prio);
 #else
  #ifdef _OS_ATMega128_
   U08      OS_TaskCreate(void (OS_FAR *ttask)(void *dptr), void *tdata, void *pstk, U16 stksize, U08 prio);
  #else
   #if OS_STK_CHECK_EN
    U08     OS_TaskCreate(void (OS_FAR *ttask)(void *dptr), void *tdata, void *pstk, void *pstkend, U08 prio);
   #else
    U08     OS_TaskCreate(void (OS_FAR *ttask)(void *dptr), void *tdata, void *pstk, U08 prio);
   #endif
  #endif
 #endif
#endif
void        OS_TimeDly(U16 ticks) KC51_REENTRANT;
U08         OS_TimeDlyResume(U08 prio) KC51_REENTRANT;
U08         OS_TimeDlyIdResume(U08 id) KC51_REENTRANT;
void        OS_TimeSet(U32 time) KC51_REENTRANT;
U32         OS_TimeGet(void) KC51_REENTRANT;
U08         OS_ChangePrio(U08 newp) KC51_REENTRANT;
U08         OS_TaskChangePrio(U08 oldp, U08 newp) KC51_REENTRANT;
U08         OS_TaskDelete(U08 prio) KC51_REENTRANT;
U08         OS_TaskIdDelete(U08 id) KC51_REENTRANT;
U08         OS_TaskIdDestroy(U08 id) KC51_REENTRANT;
U08         OS_TaskSuspend(U08 prio) KC51_REENTRANT;
U08         OS_TaskIdSuspend(U08 id) KC51_REENTRANT;
U08         OS_TaskResume(U08 prio) KC51_REENTRANT;
U08         OS_TaskIdResume(U08 id) KC51_REENTRANT;
U08         OS_TaskGetPrio(U08 id) KC51_REENTRANT;
U08         OS_TaskGetID(U08 prio) KC51_REENTRANT;
U08         OS_TaskGetStatus(U08 prio) KC51_REENTRANT;
U08         OS_TaskIdGetStatus(U08 id) KC51_REENTRANT;

void        OS_Lock(void) KC51_REENTRANT;
void        OS_Unlock(void) KC51_REENTRANT;
void        OSIntEnter(void);
void        OSIntExit(void);

void        OSSystemTick(void);
void OS_FAR OSCtxSw(void);
void        OSIntCtxSw(void);
void OS_FAR OSTickISR(void);
void        OSStartHighRdy(void);
void        OSSched(void) KC51_REENTRANT;
OS_TCB     *OSTCBGetFree(void) KC51_REENTRANT;
#if OS_STK_CHECK_EN
 U08        OSTCBInit(U08 p, void OS_FAR *pstk, void OS_FAR *pstkend) KC51_REENTRANT;
#else
 U08        OSTCBInit(U08 p, void OS_FAR *pstk) KC51_REENTRANT;
#endif

U08         OS_SemInit(OS_SEM *psem, U16 value) KC51_REENTRANT;
U08         OS_SemClear(OS_SEM *psem) KC51_REENTRANT;
U08         OS_SemPost(OS_SEM *psem) KC51_REENTRANT;
U08         OS_SemPend(OS_SEM *psem, U16 timeout) KC51_REENTRANT;
U08         OS_SemPendAbbort(OS_SEM *psem) KC51_REENTRANT;
U08         OS_SemAccept(OS_SEM *psem, U16 *cnt, U16 timeout) KC51_REENTRANT;

U08         OS_MutexCreate(OS_MUX *pmux, U08 p) KC51_REENTRANT;
U08         OS_MutexPost(OS_MUX *pmux) KC51_REENTRANT;
U08         OS_MutexPend(OS_MUX *pmux, U16 timeout) KC51_REENTRANT;
U08         OS_MutexPendAbbort(OS_MUX *pmux) KC51_REENTRANT;

U08         OS_MboxInit(OS_MBOX *pmbox) KC51_REENTRANT;
U08         OS_MboxPost(OS_MBOX *pmbox, void OS_FAR *msg, U16 timeout) KC51_REENTRANT;
U08         OS_MboxPostAbbort(OS_MBOX *pmbox) KC51_REENTRANT;
U08         OS_MboxPend(OS_MBOX *pmbox, void OS_FAR **msg, U16 timeout) KC51_REENTRANT;
U08         OS_MboxPendAbbort(OS_MBOX *pmbox) KC51_REENTRANT;

U08         OS_QueueInit(OS_Q *pq, void OS_HUGE *Qdata, U16 size) KC51_REENTRANT;
U08         OS_QueueInfo(OS_Q *pq, U16 *size, U16 *used, U08 *prio) KC51_REENTRANT;
U08         OS_QueueClear(OS_Q *pq) KC51_REENTRANT;
U08         OS_QueuePost(OS_Q *pq, U08 msg, U16 timeout) KC51_REENTRANT;
U08         OS_QueueFrontPost(OS_Q *pq, U08 msg, U16 timeout) KC51_REENTRANT;
U08         OS_QueuePostAbbort(OS_Q *pq) KC51_REENTRANT;
U08         OS_QueuePend(OS_Q *pq, U08 OS_FAR *msg, U16 timeout) KC51_REENTRANT;
U08         OS_QueuePendAbbort(OS_Q *pq) KC51_REENTRANT;

U08         OS_PipeInit(OS_P *pp, void OS_HUGE *Pdata, U16 size, U08 deep) KC51_REENTRANT;
U08         OS_PipeInfo(OS_P *pp, U16 *size, U08 *deep, U08 *used, U08 *prio) KC51_REENTRANT;
U08         OS_PipeClear(OS_P *pp) KC51_REENTRANT;
U08         OS_PipePost(OS_P *pp, U08 OS_HUGE *msg, U16 lenght, U16 timeout) KC51_REENTRANT;
U08         OS_PipeFrontPost(OS_P *pp, U08 OS_HUGE *msg, U16 lenght, U16 timeout) KC51_REENTRANT;
U08         OS_PipePostAbbort(OS_P *pp) KC51_REENTRANT;
U08         OS_PipePend(OS_P *pp, U08 OS_HUGE *msg, U16 *lenght, U16 timeout) KC51_REENTRANT;
U08         OS_PipePendAbbort(OS_P *pp) KC51_REENTRANT;

U08         OS_EvgInit(OS_EVG *pevg) KC51_REENTRANT;
U08         OS_EvgPost(OS_EVG *pevg, U32 events, U08 mode) KC51_REENTRANT;
U08         OS_EvgPend(OS_EVG *pevg, U32 *events, U08 mode, U16 timeout) KC51_REENTRANT;
U08         OS_EvgPendAbbort(OS_EVG *pevg) KC51_REENTRANT;

U08         OS_TimerCreate(OS_TMR *ptmr, U32 time, void(*tFct)(void *), void *tArg, U08 mode) KC51_REENTRANT;
U08         OS_TimerDelete(OS_TMR *ptmr) KC51_REENTRANT;                         
U08         OS_TimerStart(OS_TMR *ptmr, U32 time) KC51_REENTRANT;                         
U08         OS_TimerStop(OS_TMR *ptmr) KC51_REENTRANT;                         
U08         OS_TimerGetState(OS_TMR *ptmr) KC51_REENTRANT;
U32         OS_TimerGetRemain(OS_TMR *ptmr) KC51_REENTRANT;
U08         OSTimerTick(void);

U08         OS_MemoryInit(void OS_HUGE *mp, U32 size) KC51_REENTRANT;
U08         OS_MemAlloc(void OS_HUGE *(*MemPtr), U32 size) KC51_REENTRANT;
U08         OS_MemFree(void OS_HUGE *MemPtr) KC51_REENTRANT;
U32         OS_MemFreeSize(void) KC51_REENTRANT;

U08         OS_HistoryPost(U32 param1, U32 param2) KC51_REENTRANT;
U08         OS_HistoryRead(U32 *param1, U32 *param2, U08 *prio, U32 *time) KC51_REENTRANT;

/************************** END ***************************/
#endif



