/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_8051_c.c                         *
*                  8051 Specific Code                      *
*                  LARGE MEMORY MODEL                      *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
U08 OS_TaskCreate(void (*task)(void *pd), void *Tdata, void *pstk, U16 stksize, U08 p) KC51_REENTRANT
{
    U08 *stk, *estk;
    U08  err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                       /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk      = (U08 *)pstk; 
        estk     = (U08 *)&stk[stksize];          
#ifdef __C51__                                                  /* KEIL C51 with Generic pointer        */
        *stk     = (U08)(task);                                 /* PCL                                  */
        *++stk   = (U08)((U16)task >>8 );                       /* PCH                                  */
        *++stk   = 0x00;                                        /* PSW                                  */
        *++stk   = 0x00;                                        /* ACC                                  */
        *++stk   = 0x00;                                        /* B                                    */
        *++stk   = 0x00;                                        /* R0                                   */
        *++stk   = (U08)(Tdata);                                /* R1 data LSB                          */
        *++stk   = (U08)((U16)Tdata >>8 );                      /* R2 data MSB                          */
        *++stk   = (U08)((U32)Tdata >>16 );                     /* R3 data G                            */
        *++stk   = 0x04;                                        /* R4                                   */
        *++stk   = 0x05;                                        /* R5                                   */
        *++stk   = 0x06;                                        /* R6                                   */
        *++stk   = 0x07;                                        /* R7                                   */
        *++stk   = 0x00;                                        /* DPH                                  */
        *++stk   = 0x00;                                        /* DPL                                  */
        *++stk   = (U08)((U16)estk >>8 );                       /* XSP area MSB                         */
        *++stk   = (U08)(estk);                                 /* XSP area LSB                         */
        *++stk   = 18;                                          /* SP (bytes)                           */
#else
#error This piece of code only tested with Keil C51
#endif
        err = OSTCBInit(p, (void OS_FAR *)stk);
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

