/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                    OS_ARC_EM_c.c                         *
*                 ARC EM Specific Code                     *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U32 *stk;
    U08  err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk      = (U32 *)pstk;                               /* decrement first !                                  */
        *--stk   = (U32)0L;                                   /* "__Xstack_check" space (excluding STACK_BASE)      */

#ifndef __Xsec_modes                                          /* =============== no SecureShield ================== */

        *--stk   = (read_aux_reg(ARC_REG_STATUS32) & 0x1E)    /* STATUS32       : Status Register                   */
 #if OS_STK_CHECK_EN && __Xstack_check
		                        | STATUS_SC_MASK
 #endif
		                        | STATUS_IE_MASK;
        *--stk   = (U32)(task);                               /* PC             : Program Counter                   */

        *--stk   = (U32)0L;                                   /* JLI_BASE       : Jump & Link Indexed Base Address  */
        *--stk   = (U32)0L;                                   /* LDI_BASE       : Load Indexed Base Address         */
        *--stk   = (U32)0L;                                   /* EI_BASE        : Execute Indexed Base Address      */

        *--stk   = (U32)0L;                                   /* r60 / LP_COUNT : Loop Count Register               */
        *--stk   = (U32)0L;                                   /* LP_START       : Loop Start Register               */
        *--stk   = (U32)0L;                                   /* LP_END         : Loop End Register                 */

        *--stk   = (U32)(task);                               /* r31  / blink   : Branch link register              */
        *--stk   = (U32)0L;                                   /* r30            :                                   */
        *--stk   = (U32)(task);                               /* r29  / ilink   : Interrupt link register           */
        *--stk   = (U32)(pstk);                               /* r28  / sp      : Stack pointer                     */

        *--stk   = (U32)0L;                                   /* r27  / fp      : Frame Pointer                     */
        *--stk   = (U32)_core_read(26);                       /* r26  / gp      : Global Pointer                    */
 #ifndef _ARC_RF16
        *--stk   = (U32)0x25252525L;                          /* r25            : Basecase Core Register            */
        *--stk   = (U32)0x24242424L;                          /* r24            : Basecase Core Register            */
        *--stk   = (U32)0x23232323L;                          /* r23            : Basecase Core Register            */
        *--stk   = (U32)0x22222222L;                          /* r22            : Basecase Core Register            */
        *--stk   = (U32)0x21212121L;                          /* r21            : Basecase Core Register            */
        *--stk   = (U32)0x20202020L;                          /* r20            : Basecase Core Register            */
        *--stk   = (U32)0x19191919L;                          /* r19            : Basecase Core Register            */
        *--stk   = (U32)0x18181818L;                          /* r18            : Basecase Core Register            */
        *--stk   = (U32)0x17171717L;                          /* r17            : Basecase Core Register            */
        *--stk   = (U32)0x16161616L;                          /* r16            : Basecase Core Register            */
 #endif
        *--stk   = (U32)0x15151515L;                          /* r15            : Basecase Core Register            */
        *--stk   = (U32)0x14141414L;                          /* r14            : Basecase Core Register            */
        *--stk   = (U32)0x13131313L;                          /* r13            : Basecase Core Register            */
        *--stk   = (U32)0x12121212L;                          /* r12            : Basecase Core Register            */
        *--stk   = (U32)0x11111111L;                          /* r11            : Basecase Core Register            */
        *--stk   = (U32)0x10101010L;                          /* r10            : Basecase Core Register            */
 #ifndef _ARC_RF16
        *--stk   = (U32)0x09090909L;                          /* r9             : Basecase Core Register            */
        *--stk   = (U32)0x08080808L;                          /* r8             : Basecase Core Register            */
        *--stk   = (U32)0x07070707L;                          /* r7             : Basecase Core Register            */
        *--stk   = (U32)0x06060606L;                          /* r6             : Basecase Core Register            */
        *--stk   = (U32)0x05050505L;                          /* r5             : Basecase Core Register            */
        *--stk   = (U32)0x04040404L;                          /* r4             : Basecase Core Register            */
 #endif
        *--stk   = (U32)0x03030303L;                          /* r3             : Basecase Core Register            */
        *--stk   = (U32)0x02020202L;                          /* r2             : Basecase Core Register            */
        *--stk   = (U32)0x01010101L;                          /* r1             : Basecase Core Register            */
        *--stk   = (U32)Tdata;                                /* r0             : argument                          */

#else                                                         /* ============== with SecureShield ================= */

        *--stk   = (read_aux_reg(ARC_REG_STATUS32) & 0x1E)    /* STATUS32       : Status Register                   */
 #if OS_STK_CHECK_EN && __Xstack_check
		                        | STATUS_SC_MASK
 #endif
		                        | STATUS_IE_MASK;
        *--stk   = (U32)0L;                                   /* SEC_STAT       : Secure Status Register            */
        *--stk   = (U32)(task);                               /* PC             : Program Counter                   */

        *--stk   = (U32)(task);                               /* r31  / blink   : Branch link register              */
        *--stk   = (U32)0L;                                   /* r30            :                                   */
        *--stk   = (U32)(task);                               /* r29  / ilink   : Interrupt link register           */
        *--stk   = (U32)(pstk);                               /* r28  / sp      : Stack pointer                     */
        *--stk   = (U32)0L;                                   /* r27  / fp      : Frame Pointer                     */
        *--stk   = (U32)_core_read(26);                       /* r26  / gp      : Global Pointer                    */

 #ifndef _ARC_RF16
        *--stk   = (U32)0x25252525L;                          /* r25            : Basecase Core Register            */
        *--stk   = (U32)0x24242424L;                          /* r24            : Basecase Core Register            */
        *--stk   = (U32)0x23232323L;                          /* r23            : Basecase Core Register            */
        *--stk   = (U32)0x22222222L;                          /* r22            : Basecase Core Register            */
        *--stk   = (U32)0x21212121L;                          /* r21            : Basecase Core Register            */
        *--stk   = (U32)0x20202020L;                          /* r20            : Basecase Core Register            */
        *--stk   = (U32)0x19191919L;                          /* r19            : Basecase Core Register            */
        *--stk   = (U32)0x18181818L;                          /* r18            : Basecase Core Register            */
        *--stk   = (U32)0x17171717L;                          /* r17            : Basecase Core Register            */
        *--stk   = (U32)0x16161616L;                          /* r16            : Basecase Core Register            */
 #endif
        *--stk   = (U32)0x15151515L;                          /* r15            : Basecase Core Register            */
        *--stk   = (U32)0x14141414L;                          /* r14            : Basecase Core Register            */
        *--stk   = (U32)0x13131313L;                          /* r13            : Basecase Core Register            */
        *--stk   = (U32)0x12121212L;                          /* r12            : Basecase Core Register            */
        *--stk   = (U32)0x11111111L;                          /* r11            : Basecase Core Register            */
        *--stk   = (U32)0x10101010L;                          /* r10            : Basecase Core Register            */
 #ifndef _ARC_RF16
        *--stk   = (U32)0x09090909L;                          /* r9             : Basecase Core Register            */
        *--stk   = (U32)0x08080808L;                          /* r8             : Basecase Core Register            */
        *--stk   = (U32)0x07070707L;                          /* r7             : Basecase Core Register            */
        *--stk   = (U32)0x06060606L;                          /* r6             : Basecase Core Register            */
        *--stk   = (U32)0x05050505L;                          /* r5             : Basecase Core Register            */
        *--stk   = (U32)0x04040404L;                          /* r4             : Basecase Core Register            */
 #endif
        *--stk   = (U32)0x03030303L;                          /* r3             : Basecase Core Register            */
        *--stk   = (U32)0x02020202L;                          /* r2             : Basecase Core Register            */
        *--stk   = (U32)0x01010101L;                          /* r1             : Basecase Core Register            */
        *--stk   = (U32)Tdata;                                /* r0             : argument                          */

        *--stk   = (U32)0L;                                   /* JLI_BASE       : Jump & Link Indexed Base Address  */
        *--stk   = (U32)0L;                                   /* LDI_BASE       : Load Indexed Base Address         */
        *--stk   = (U32)0L;                                   /* EI_BASE        : Execute Indexed Base Address      */

        *--stk   = (U32)0L;                                   /* r60 / LP_COUNT : Loop Count Register               */
        *--stk   = (U32)0L;                                   /* LP_START       : Loop Start Register               */
        *--stk   = (U32)0L;                                   /* LP_END         : Loop End Register                 */

#endif                                                        /* ================================================== */

        // PUSH user extension register startup values here, like DSP, FPU, ...
        // ( same as in assembler macro "store_extensions" )

#if OS_STK_CHECK_EN && __Xstack_check
        *--stk   = (U32)pstkend;                              /* STACK_TOP                                          */
        *--stk   = (U32)pstk;                                 /* STACK_BASE                                         */
#endif

#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

