//***************************************************************
//*                       pC/OS 1.70b                           *
//*                      a FREE - RTOS                          *
//*                                                             *
//*                 based on the C/OS 1.00                     *
//*                   by  Jean J. Labrosse                      *
//*                                                             *
//*    changed by Frank Goetze  -  www.embedded-os.de           *
//***************************************************************
//*                    OS_AT91SAM7xxx_a.s                       *
//*                  AT91-ARM Specific Code                     *
//*                     using CrossWorks                        *
//***************************************************************
        .extern  OSTCBCur
        .extern  OSTCBHighRdy
        .extern  OSIntExit
        .extern  OSIntNesting

        .equ     NO_INT,     0xC0                    // mask to disable interrupts
        .equ     FIQ32_MODE, 0x11
        .equ     IRQ32_MODE, 0x12
        .equ     SVC32_MODE, 0x13

        .equ     AIC_IVR,    0xFFFFF100              // IRQ vector register
        .equ     AIC_FVR,    0xFFFFF104              // FIQ vector register
        .equ     AIC_EOICR,  0xFFFFF130              // EOI command register

        .text
//***************************************************************
//                     START MULTITASKING
//                  void OSStartHighRdy(void)
//***************************************************************
        .global OSStartHighRdy
        .code 32
OSStartHighRdy:
        MSR     CPSR_cxsf, #0xD3            // switch to SVC mode with IRQ and FIQ disabled
        LDR     R6, =OSTCBHighRdy           // get TCB of highest priority task
        LDR     R4, =OSTCBCur               // save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    // restore stack-ptr
        LDR     R4,  [SP], #4               // pop task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        // pop task context

//***************************************************************
//           PERFORM A CONTEXT SWITCH (From task level)
//                      void OSCtxSw(void)
//***************************************************************
        .global OSCtxSw
        .code 32
OSCtxSw:
        STMFD   SP!, {LR}                   // push return address
        STMFD   SP!, {LR}
        STMFD   SP!, {R0-R12}               // push registers
        MRS     R4, CPSR                    // push current CPSR
        TST     LR, #1                      // see if called from Thumb mode
        ORRNE   R4, R4, #0x20               // if yes, Set the T-bit
        STMFD   SP!, {R4}
        LDR     R4, =OSTCBCur               // Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
        LDR     R6, =OSTCBHighRdy           // get TCB of highest priority task
        LDR     R4, =OSTCBCur               // save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    // restore new stack-ptr
        LDMFD   SP!, {R4}                   // pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        // pop new task context

//***************************************************************
//           PERFORM A CONTEXT SWITCH (From an ISR)
//                    void OSIntCtxSw(void)
//***************************************************************
        .global OSIntCtxSw
        .code 32
                                            // no TCB-save since the INTs can cascaded
OSIntCtxSw:                                 // (only saved on first INT by OSirqISR or OSfiqISR)
        LDR     R6, =OSTCBHighRdy           // get TCB of highest priority task
        LDR     R4, =OSTCBCur               // save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    // restore new stack-ptr
        LDMFD   SP!, {R4}                   // pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        // pop new task context

//***************************************************************
//                       HANDLE TICK ISR
//                     void OSTickISR(void)
//***************************************************************
//  ==> this is not asm-relevant for this processor
//      the C-TimerISR is to hang into the table of
//      the following IRQ-isr
//***************************************************************


//***************************************************************
//                    SPECIAL IRQ-ISR (ALL)
//                     void OSirqISR(void)
//***************************************************************
        .global OSirqISR
        .code 32
OSirqISR:
        STMFD   SP!, {R1-R3}                // push working registers onto IRQ stack
        MOV     R1, SP                      // save IRQ stack pointer
        ADD     SP, SP,#12                  // adjust IRQ stack pointer
        SUB     R2, LR,#4                   // adjust PC for return address to task
        MRS     R3, SPSR                    // copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) // change to SVC mode
        STMFD   SP!, {R2}                   // push tasks return PC
        STMFD   SP!, {LR}                   // push tasks LR
        STMFD   SP!, {R4-R12}               // push tasks R12-R4
        LDMFD   R1!, {R4-R6}                // move tasks R1-R3 from IRQ stack to SVC stack
        STMFD   SP!, {R4-R6}
        STMFD   SP!, {R0}                   // push task R0 onto task stack
        STMFD   SP!, {R3}                   // push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           // OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R1, [R0]
        ADD     R1, R1,#1
        STRB    R1, [R0]
        CMP     R1, #1                      // if OSIntNesting == 1 ...
        BNE     OSirqISR1
        LDR     R4, =OSTCBCur               // ... Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
OSirqISR1:
        MSR     CPSR_c, #(NO_INT | IRQ32_MODE) // change to IRQ mode (to use the IRQ stack to handle interrupt)

        LDR     R1, =AIC_IVR                // branch to the routine pointed by the AIC_IVR
        LDR     R0, [R1]
        STR     R1, [R1]
        MOV     LR, PC
        BX      R0

        MSR     CPSR_c, #(NO_INT | SVC32_MODE) // change to SVC mode
        LDR     R0, =AIC_EOICR              // mark the End-Of-Interrupt on the AIC
        STR     R0, [R0]
        LDR     R0, =OSIntExit              // call OSIntExit() -> perform a context-switch if nesesary
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R4}                   // pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        // pop new task context

//***************************************************************
//                    SPECIAL FIQ-ISR (ALL)
//                     void OSfiqISR(void)
//***************************************************************
        .global OSfiqISR
        .code 32
OSfiqISR:
        STMFD   SP!, {R1-R3}                // push working registers onto FIQ stack
        MOV     R1, SP                      // save FIQ stack pointer
        ADD     SP, SP,#12                  // adjust FIQ stack pointer
        SUB     R2, LR,#4                   // adjust PC for return address to task
        MRS     R3, SPSR                    // copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) // change to SVC mode
        STMFD   SP!, {R2}                   // push tasks return PC
        STMFD   SP!, {LR}                   // push tasks LR
        STMFD   SP!, {R4-R12}               // push tasks R12-R4
        LDMFD   R1!, {R4-R6}                // move tasks R1-R3 from FIQ stack to SVC stack
        STMFD   SP!, {R4-R6}
        STMFD   SP!, {R0}                   // push task R0 onto task stack
        STMFD   SP!, {R3}                   // push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           // OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R1, [R0]
        ADD     R1, R1,#1
        STRB    R1, [R0]
        CMP     R1, #1                      // if OSIntNesting == 1
        BNE     OSfiqISR1
        LDR     R4, =OSTCBCur               // Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
OSfiqISR1:
        MSR     CPSR_c, #(NO_INT | FIQ32_MODE) // change to FIQ mode (to use the IRQ stack to handle interrupt)

        LDR     R1, =AIC_FVR                // branch to the routine pointed by the AIC_FVR
        LDR     R0, [R1]
        STR     R1, [R1]
        MOV     LR, PC
        BX      R0

        MSR     CPSR_c, #(NO_INT | SVC32_MODE) // change to SVC mode

// end-of-interrupt to add !

        LDR     R0, =OSIntExit              // call OSIntExit() -> perform a context-switch if nesesary
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R4}                   // pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        // pop new task context

//***************************************************************
//                    SPECIAL INT-SAVE CALLS
//                  OS_STK_TYPE OSEnterCrit(void)
//                  void OSExitCrit(OS_STK_TYPE)
//***************************************************************
        .global OSEnterCrit
        .code 32
OSEnterCrit:
        MRS     R0, CPSR
        ORR     R1, R0, #NO_INT
        MSR     CPSR_c, R1
        ORR     R1, R1, #NO_INT
        MSR     CPSR_c, R1
        BX      LR                          // return R0

        .global OSExitCrit
        .code 32
OSExitCrit:
        MSR     CPSR_c, R0
        BX      LR                          // return


//***************************************************************




