;****************************************************************
;*                       pC/OS 1.83a                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                    OS_AT91SAM7xxx_a.s                        *
;*                  AT91-ARM Specific Code                      *
;*                      using Keil-RDS                          *
;****************************************************************
NESTED_IRQ  EQU     1


    IMPORT OSTCBCur
    IMPORT OSTCBHighRdy
    IMPORT OSIntExit
    IMPORT OSIntNesting

NO_INT      EQU     0xC0                    ; interrupts
NO_IRQ      EQU     0x80
NO_FIQ      EQU     0x40

FIQ32_MODE  EQU     0x11                    ; modes
IRQ32_MODE  EQU     0x12
SVC32_MODE  EQU     0x13
SYS32_MODE  EQU     0x1F

AIC_IVR     EQU     0xFFFFF100              ; IRQ vector register
AIC_FVR     EQU     0xFFFFF104              ; FIQ vector register
AIC_EOICR   EQU     0xFFFFF130              ; EOI command register

        AREA PCOS, CODE, READONLY, ALIGN=4
        PRESERVE8
;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
    EXPORT OSStartHighRdy
OSStartHighRdy
        MSR     CPSR_cxsf, #0xD3            ; switch to SVC mode with IRQ and FIQ disabled
        LDR     R1, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R0, =OSTCBCur               ; save as current
        LDR     R1, [R1]
        STR     R1, [R0]
        LDR     SP, [R1]                    ; restore stack-ptr
        LDR     R0, [SP], #4                ; pop task CPSR
        MSR     SPSR_cxsf, R0
        LDMFD   SP!, {R0-R12, LR, PC}^      ; pop task context

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
    EXPORT OSCtxSw
OSCtxSw
        STMFD   SP!, {LR}                   ; push return address
        STMFD   SP!, {R0-R12, LR}           ; push LR and registers
        MRS     R0, CPSR                    ; push current CPSR
        TST     LR, #1                      ; see if called from Thumb mode
        ORRNE   R0, R0, #0x20               ; if yes, set the T-bit
        STMFD   SP!, {R0}
        LDR     R0, =OSTCBCur               ; save stack ptr in TCB
        LDR     R1, [R0]
        STR     SP, [R1]
        LDR     R1, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R0, =OSTCBCur               ; save as current
        LDR     R1, [R1]
        STR     R1, [R0]
        LDR     SP, [R1]                    ; restore new stack-ptr
        LDMFD   SP!, {R0}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R0
        LDMFD   SP!, {R0-R12, LR, PC}^      ; pop new task context

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
    EXPORT OSIntCtxSw
OSIntCtxSw
        LDR     R1, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R0, =OSTCBCur               ; save as current
        LDR     R1, [R1]
        STR     R1, [R0]
        LDR     SP, [R1]                    ; restore new stack-ptr
        LDMFD   SP!, {R0}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R0
        LDMFD   SP!, {R0-R12, LR, PC}^      ; pop new task context

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
;   ==> this is not asm-relevant for this processor
;       the C-TimerISR is to hang into the table of
;       the following IRQ-isr
;****************************************************************


;****************************************************************
;                     SPECIAL IRQ-ISR (ALL)
;                      void OSirqISR(void)
;****************************************************************
    EXPORT OSirqISR
OSirqISR
    IF  NESTED_IRQ != 0
        STMFD   SP!, {R1-R3}                ; push working registers onto IRQ stack
        LDR     R1, =OSIntNesting           ; OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R2, [R1]
        ADD     R2, R2, #1
        STRB    R2, [R1]
        CMP     R2, #1                      ; if (OSIntNesting == 1) save task-context in TCB
        BNE     OSirqISRnested              ; or jump to nested interrupt
        MOV     R1, SP                      ; save IRQ stack pointer for copy R1-R3
        ADD     SP, SP, #12                 ; adjust IRQ stack pointer
        SUB     R2, LR, #4                  ; adjust PC for return address to task and save
        MRS     R3, SPSR                    ; copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT :OR: SVC32_MODE) ; change to SVC mode (to store task TCB)
        STMFD   SP!, {R2}                   ; push tasks return PC
        STMFD   SP!, {LR}                   ; push tasks LR
        STMFD   SP!, {R4-R12}               ; push tasks R12-R4
        LDMFD   R1!, {R4-R6}                ; move tasks R3-R1 from IRQ stack ...
        STMFD   SP!, {R4-R6}                ; ... to SVC stack
        STMFD   SP!, {R0}                   ; push task R0 onto task stack
        STMFD   SP!, {R3}                   ; push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSTCBCur               ; save stack ptr in TCB
        LDR     R1, [R0]
        STR     SP, [R1]
        LDR     LR, =AIC_IVR                ; (AT91SAM7..) get the routine pointed by the AIC_IVR
        LDR     R0, [LR]
        MSR     CPSR_c, #(SYS32_MODE)       ; change to SYS mode (to use the SYS stack to handle interrupt)
        MOV     LR, PC                      ; save return address
        BX      R0                          ; ---> call IRQ handler
        MSR     CPSR_c, #(NO_INT :OR: SVC32_MODE) ; change back to SVC mode
        LDR     R0, =AIC_EOICR              ; (AT91SAM7..) mark the End-Of-Interrupt on the AIC
        STR     R0, [R0]
        LDR     R0, =OSIntExit              ; call OSIntExit() -> perform a context-switch if nesesary
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R0}                   ; pop task CPSR
        MSR     SPSR_cxsf, R0
        LDMFD   SP!, {R0-R12, LR, PC}^      ; pop task context

OSirqISRnested
        LDMFD   SP!, {R1-R3}                ; restore working registers from IRQ stack
        SUB     LR, LR , #4                 ; adjust PC for return address to nested interrupt
        STMFD   SP!, {LR}                   ; push nested interrupt return PC
        MRS     LR, SPSR                    ; get SPSR (i.e. interrupted interrupt CPSR)
        STMFD   SP!, {R0, LR}               ; push nested interrupt CPSR and R0
        LDR     LR, =AIC_IVR                ; (AT91SAM7..) get the routine pointed by the AIC_IVR
        LDR     R0, [LR]
        MSR     CPSR_c, #(SYS32_MODE)       ; change to SYS mode (to use the SYS stack to handle interrupt)
        STMFD   SP!, {R1-R12, LR}           ; push LR and registers onto SYS stack
        MOV     LR, PC                      ; save return address
        BX      R0                          ; ---> call IRQ handler
        LDMFD   SP!, {R1-R12, LR}           ; pop LR and registers from SYS stack
        MSR     CPSR_c, #(NO_INT :OR: IRQ32_MODE) ; change to IRQ mode
        LDR     R0, =AIC_EOICR              ; (AT91SAM7..) mark the End-Of-Interrupt on the AIC
        STR     R0, [R0]
        LDMFD   SP!, {R0, LR}               ; pop nested interrupt CPSR and R0
        MSR     SPSR_cxsf, LR
        LDMFD   SP!, {PC}^                  ; pop nested interrupt return address in PC
    ELSE
        STMFD   SP!, {R1-R3}                ; push working registers onto IRQ stack
        MOV     R1, SP                      ; save IRQ stack pointer
        ADD     SP, SP, #12                 ; adjust IRQ stack pointer
        SUB     R2, LR, #4                  ; adjust PC for return address to task
        MRS     R3, SPSR                    ; copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT :OR: SVC32_MODE) ; change to SVC mode
        STMFD   SP!, {R2}                   ; push tasks return PC
        STMFD   SP!, {LR}                   ; push tasks LR
        STMFD   SP!, {R4-R12}               ; push tasks R12-R4
        LDMFD   R1!, {R4-R6}                ; move tasks R3-R1 from IRQ stack ...
        STMFD   SP!, {R4-R6}                ; ... to SVC stack
        STMFD   SP!, {R0}                   ; push task R0 onto task stack
        STMFD   SP!, {R3}                   ; push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           ; OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R1, [R0]
        ADD     R1, R1, #1
        STRB    R1, [R0]
        CMP     R1, #1                      ; if (OSIntNesting == 1) ...
        BNE     OSirqISR1
        LDR     R0, =OSTCBCur               ; ... save stack ptr in TCB
        LDR     R1, [R0]
        STR     SP, [R1]
OSirqISR1
        LDR     R1, =AIC_IVR                ; (AT91SAM7..) branch to the routine pointed by the AIC_IVR
        LDR     R0, [R1]
        MSR     CPSR_c, #(NO_IRQ :OR: SYS32_MODE) ; change to SYS mode (to use the SYS stack to handle interrupt)
        MOV     LR, PC                      ; save return address
        BX      R0                          ; ---> call IRQ handler
        MSR     CPSR_c, #(NO_INT :OR: SVC32_MODE) ; change to SVC mode
        LDR     R0, =AIC_EOICR              ; (AT91SAM7..) mark the End-Of-Interrupt on the AIC
        STR     R0, [R0]
        LDR     R0, =OSIntExit              ; call OSIntExit() -> perform a context-switch if nesesary
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R0}                   ; pop task CPSR
        MSR     SPSR_cxsf, R0
        LDMFD   SP!, {R0-R12, LR, PC}^      ; pop task context
    ENDIF

;****************************************************************
;                     SPECIAL FIQ-ISR (ALL)
;                      void OSfiqISR(void)
;                        !!! WARNING !!!
;       the FIQ is a unmanaged but registered interrupt
;  so the called handler can use the kernel IPCs but sheduling
;  will executed only on next none-FIQ event (IPC or OSSysTick)
;****************************************************************
    EXPORT OSfiqISR
OSfiqISR
        SUB     LR, LR, #4                  ; adjust PC for return address to task or IRQ
        STMFD   SP!, {R0-R7, LR}            ; push users working registers onto FIQ stack (for C-code handler)
        LDR     R8, =OSIntNesting           ; OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R9, [R8]
        ADD     R9, R9, #1
        STRB    R9, [R8]

        LDR     R8, =AIC_FVR                ; (AT91SAM7..) branch to the routine pointed by the AIC_FVR
        LDR     R8, [R8]

        MOV     LR, PC                      ; save return address
        BX      R8                          ; ---> call FIQ handler
        LDR     R8, =OSIntNesting           ; OSIntNesting-- (don't call OSIntExit !)
        LDRB    R9, [R8]
        SUB     R9, R9, #1
        STRB    R9, [R8]
        LDMFD   SP!, {R0-R7, PC}^           ; pop users working registers and return to PC

;****************************************************************
;                     SPECIAL INT-SAVE CALLS
;                   OS_STK_TYPE OSEnterCrit(void)
;                   void OSExitCrit(OS_STK_TYPE)
;****************************************************************
    EXPORT OSEnterCritFull
OSEnterCritFull
        MRS     R0, CPSR
        ORR     R1, R0, #NO_IRQ
        MSR     CPSR_c, R1
        ORR     R1, R1, #NO_FIQ
        MSR     CPSR_c, R1
        BX      LR                          ; return R0

    EXPORT OSEnterCrit
OSEnterCrit
        MRS     R0, CPSR
        ORR     R1, R0, #NO_IRQ
        MSR     CPSR_c, R1
        ORR     R1, R1, #NO_IRQ
        MSR     CPSR_c, R1
        BX      LR                          ; return R0

    EXPORT OSExitCrit
OSExitCrit
        MSR     CPSR_c, R0
        BX      LR                          ; return

    EXPORT OSEnableInts
OSEnableInts
        MRS     R0, CPSR
        AND     R0, R0, #0x3F               ; allow all interrupts
        MSR     CPSR_c, R0
        BX      LR                          ; return

;****************************************************************
        END


