#***************************************************************
#*                       pC/OS 1.88c                           *
#*                      a FREE - RTOS                          *
#*                                                             *
#*                 based on the C/OS 1.00                     *
#*                   by  Jean J. Labrosse                      *
#*                                                             *
#*    changed by Frank Goetze  -  www.embedded-os.de           *
#***************************************************************
#*                      OS_MPC5744_a.s                         *
#*         Freescale e200z MPC5744 VLE Specific Code           *
#*                         using S32                           *
#***************************************************************

#include "OS_MPC5744_a.inc"

    .equ INTC_IACKR, 0xfc040020
    .equ INTC_EOIR,  0xfc040030

        .text 

#***************************************************************

    .extern OSTCBCur
    .extern OSTCBHighRdy
    .extern OSIntNesting
    .extern OSTickISR
    .extern OSIntExit


#**************************************************************
#                     START MULTITASKING
#                  void OSStartHighRdy(void)
#***************************************************************
    .global OSStartHighRdy
    .align  16 
OSStartHighRdy:
        e_lis     r11, OSTCBHighRdy@ha
        e_lwz     r11, OSTCBHighRdy@l(r11)
        e_lwz     r1,  0(r11)
        _OS_Restore_CPU_Context
        se_rfi

#***************************************************************
#           PERFORM A CONTEXT SWITCH (From task level)
#                      void OSCtxSw(void)
#***************************************************************
#   not needed here
#***************************************************************

#***************************************************************
#                    HANDLE CONTEXT-SWITCH
#                   void OSCtxSw_ISR(void)
#***************************************************************
#   ==> this must be hanged into core exceptions IVOR8
#***************************************************************
    .global OSCtxSw_ISR
    .align  16
OSCtxSw_ISR:
        _OS_Save_CPU_Context
        e_lis     r11, OSTCBCur@ha
        e_lwz     r3,  OSTCBCur@l(r11)
        e_stw     r1,  0(r3)
        e_lis     r12, OSTCBHighRdy@ha
        e_lwz     r12, OSTCBHighRdy@l(r12)
        e_stw     r12, OSTCBCur@l(r11)
        e_lwz     r1,  0(r12)
        _OS_Restore_CPU_Context
        se_rfi
        
#***************************************************************
#           PERFORM A CONTEXT SWITCH (From an ISR)
#                    void OSIntCtxSw(void)
#***************************************************************
    .global OSIntCtxSw
    .align  16 
OSIntCtxSw: 
        mbar    1
        e_lis     r11, OSTCBHighRdy@ha
        e_lwz     r11, OSTCBHighRdy@l(r11)
        e_lis     r12, OSTCBCur@ha
        e_stw     r11, OSTCBCur@l(r12)
        e_lwz     r1,  0(r11)
        _OS_Restore_CPU_Context
        mbar    1
        se_rfi

#***************************************************************
#                       HANDLE TICK ISR
#                     void OSTick_ISR(void)
#***************************************************************
#   not needed here - the MPC5744P does'nt have a DEC
#***************************************************************

#***************************************************************
#              SPECIAL IRQ-ISR (all external interrupts)
#                     void OSExtInt_ISR(void)
#***************************************************************
#   ==> this must be hanged into core exceptions IVOR4
#***************************************************************
    .global OSExtInt_ISR
    .align  16
OSExtInt_ISR:
        _OS_Save_CPU_Context
        e_lis     r11,OSIntNesting@ha
        e_lbz     r3, OSIntNesting@l(r11)
        e_addi    r0, r3, 1
        e_stb     r0, OSIntNesting@l(r11)
        e_cmpl16i r0,  1
        se_bne    OSExtInt_ISR_NotSaveSP
        e_lis     r11, OSTCBCur@ha
        e_lwz     r11, OSTCBCur@l(r11)
        e_stw     r1,  0(r11)
OSExtInt_ISR_NotSaveSP:
        e_lis     r3,  INTC_IACKR@ha
        e_lwz     r3,  INTC_IACKR@l(r3)
        e_lwz     r12, 0(r3)
        mtctr     r12
        se_bctrl
        se_li     r0, 0
        e_lis     r3, INTC_EOIR@h
        e_or2i    r3, INTC_EOIR@l
        e_stw     r0, 0(r3)
        e_bl      OSIntExit
        _OS_Restore_CPU_Context
        se_rfi


#***************************************************************
#                    SPECIAL INT-SAVE CALLS
#                  OS_STK_TYPE OSEnterCrit(void)
#                  void OSExitCrit(OS_STK_TYPE)
#***************************************************************
    .global OSEnterCrit
    .align  16
OSEnterCrit:
        mfmsr   r3 
        wrteei  0
        se_blr

    .global OSExitCrit
    .align  16
OSExitCrit:
        mtmsr   r3 
        se_blr

#***************************************************************
#                  SPECIAL HELPERS for MCP5744P
#***************************************************************
    .global OS_SR_Rd
    .align  16
OS_SR_Rd:
        mfmsr   r3
        se_blr

    .global IVOR1dec_ISR
    .align  16
IVOR1dec_ISR:
        mfesr     r2          # r2 - exception syndrome register            ( 62  )
        mfdc_dat  r3          # r3 - Machine Check Save/Restore Register 0  ( 570 )
        mfmcsrr1  r4          # r4 - Machine Check Save/Restore Register 1  ( 571 )
        mfmcsr    r5          # r5 - Machine Check Syndrome Register        ( 572 )
        mfmcar    r6          # r6 - Machine Check Address Register         ( 573 )
        mfmsr     r7          # r7 - Machine State Register
	    e_b     .

#***************************************************************
    .end


