/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                     OS_PIC32_c.c                         *
*            PIC32MX MIPS32-4k Specific Code               *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U32 *stk, wSR, wGP;
    U08  err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        asm volatile("mfc0  %0, $12" : "=r"(wSR));
        wSR |= 0x01;
        asm volatile("addi  %0, $28, 0" : "=r"(wGP));
        OS_EXIT_CRITICAL();
        stk      = (U32 *)pstk;                               /* decrement first !                    */
        *--stk   = (U32)(task);                               /* $31  = ra     : task start address   */
        *--stk   = (U32)0x30303030L;                          /* $30  = fp                            */
        *--stk   = (U32)wGP;                                  /* $28  = gp                            */
        *--stk   = (U32)0x27272727L;                          /* $27  = k1                            */
        *--stk   = (U32)0x26262626L;                          /* $26  = k0                            */
        *--stk   = (U32)0x25252525L;                          /* $25  = t9                            */
        *--stk   = (U32)0x24242424L;                          /* $24  = t8                            */
        *--stk   = (U32)0x23232323L;                          /* $23  = s7                            */
        *--stk   = (U32)0x22222222L;                          /* $22  = s6                            */
        *--stk   = (U32)0x21212121L;                          /* $21  = s5                            */
        *--stk   = (U32)0x20202020L;                          /* $20  = s4                            */
        *--stk   = (U32)0x19191919L;                          /* $19  = s3                            */
        *--stk   = (U32)0x18181818L;                          /* $18  = s2                            */
        *--stk   = (U32)0x17171717L;                          /* $17  = s1                            */
        *--stk   = (U32)0x16161616L;                          /* $16  = s0                            */
        *--stk   = (U32)0x15151515L;                          /* $15  = t7                            */
        *--stk   = (U32)0x14141414L;                          /* $14  = t6                            */
        *--stk   = (U32)0x13131313L;                          /* $13  = t5                            */
        *--stk   = (U32)0x12121212L;                          /* $12  = t4                            */
        *--stk   = (U32)0x11111111L;                          /* $11  = t3                            */
        *--stk   = (U32)0x10101010L;                          /* $10  = t2                            */
        *--stk   = (U32)0x09090909L;                          /* $9   = t1                            */
        *--stk   = (U32)0x08080808L;                          /* $8   = t0                            */
        *--stk   = (U32)0x07070707L;                          /* $7   = a3                            */
        *--stk   = (U32)0x06060606L;                          /* $6   = a2                            */
        *--stk   = (U32)0x05050505L;                          /* $5   = a1                            */
        *--stk   = (U32)Tdata;                                /* $4   = a0     : argument             */
        *--stk   = (U32)0x03030303L;                          /* $3   = v1                            */
        *--stk   = (U32)0x02020202L;                          /* $2   = v0                            */
        *--stk   = (U32)0x01010101L;                          /* $1   = at                            */
        *--stk   = (U32)0x00000000L;                          /* HI                                   */
        *--stk   = (U32)0x00000000L;                          /* LO                                   */
        *--stk   = (U32)(task);                               /* task start address                   */
        *--stk   = (U32)wSR;                                  /* C0 SR (HW0 = En, IE= En)             */
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

