/*************************************************************
*
* Description:
*	R32C - Example program using IAR
*
* Author[s]:
*	Frank Goetze
**************************************************************/
#include <ior32c111.h>                /* R32C peripheral library */

#define FOSC                  8000000U

#include "../../inc/pC_OS.h"

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define CORE_TICK_DIV         (FOSC / (2 * OS_TIMER_TICKS_PER_SEC))

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
OS_SEM LedSem;
OS_TMR LedTimer;
OS_TMR ROnceTimer1;
OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

int main(void)
{
    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);
#if OS_STK_CHECK_EN
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], &Stk1, 2);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], &Stk2, 4);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], &Stk3, 3);
#else
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], 2);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], 4);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], 3);
#endif
    OS_MutexCreate(&Mutex1, 1);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif
    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical PIC32 pC/OS ISR
{
#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

#pragma vector = TIMER_A2
__interrupt void ta2_isr(void)
{
    ta2s = 0;
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    ta2s  = 0;
    ta2mr = 0x80;
//    ta2   = (f1_CLK_SPEED * TIMER_PERIOD) / (32 * 1000)-1;
    ta2   = CORE_TICK_DIV;
    ta2ic = 0x02;
    ta2s  = 1;
}

void Task1(void *Tdata)
{
    U08 returnOk;
    U32 event, i;
    U08 OS_HUGE *ptr;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((OS_MEM OS_HUGE *)ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Taskswitch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                  // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);              // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);           // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif
    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
     }
}
