/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_R32C_c.c                         *
*                 R32C111 Specific Code                    *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U32 *stk, flag;
    U08  err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        flag     = 0x00000040;
        stk      = (U32 *)pstk;                               /* decrement first !                    */
/*
        *--stk   = (U32)(Tdata);
        *--stk   = (U32)(task);
*/
        *--stk   = (U32)(flag);                               /* Flag register                        */
        *--stk   = (U32)(task);                               /* PC register                          */
        *--stk   = (U32)0xFBFBFBFBL;                          /* FB register                          */
        *--stk   = (U32)0x5B5B5B5BL;                          /* SB register                          */
        *--stk   = (U32)0xA3A3A3A3L;                          /* A3 register                          */
        *--stk   = (U32)0xA2A2A2A2L;                          /* A2 register                          */
        *--stk   = (U32)0xA1A1A1A1L;                          /* A1 register                          */
#ifdef __IAR_SYSTEMS_ICC__
        *--stk   = (U32)(Tdata);
#else
        *--stk   = (U32)0xA0A0A0A0L;                          /* A0 register                          */
#endif
        *--stk   = (U32)0x77775555L;                          /* R7R5 register                        */
        *--stk   = (U32)0x66664444L;                          /* R6R4 register                        */
        *--stk   = (U32)0x33331111L;                          /* R3R1 register                        */
        *--stk   = (U32)0x22220000L;                          /* R2R0 register                        */
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

