/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                       OS_RL78_c.c                        *
*                Renesas RL78 Specific Code                *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_STK_TYPE *stk;
    U08  err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk      = (U16 *)pstk;                               /* decrement first !                    */
#if __DATA_MODEL__ != __DATA_MODEL_NEAR__
 #error "none-NEAR not fully implemented & tested!"
        *--stk   = (U16)((U32)Tdata >> 16);                   /* parameter                            */
        *--stk   = (U16)Tdata;
        *--stk   = (U16)(((U32)task >> 16) & 0x000F) | 0x8600;/* PC bits 16-19 in lower 8 bits, psw in upper 16 bits      */
        *--stk   = (U16)task;                                 /* PC bits 0-15                         */
        *--stk   = (U16)0x1100;                               /* RP0                                  */
        *--stk   = (U16)0x3322;                               /* RP1                                  */
        *--stk   = (U16)0x5544;                               /* RP2                                  */
        *--stk   = (U16)0x7766;                               /* RP3                                  */ 
#else
        *--stk   = (U16)(((U32)task >> 16) & 0x000F) | 0x8600;/* PC bits 16-19 in lower 8 bits, psw in upper 16 bits      */
        *--stk   = (U16)task;                                 /* PC bits 0-15                         */
        *--stk   = (U16)Tdata;                                /* RP0 - parameter                      */
        *--stk   = (U16)0x3322;                               /* RP1                                  */
        *--stk   = (U16)0x5544;                               /* RP2                                  */
        *--stk   = (U16)0x7766;                               /* RP3                                  */ 
        *--stk   = (U16)0x0000;
#endif

#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

/**********************************************************/
 
#ifdef __IAR_SYSTEMS_ICC__                                // IAR compilers
 #if (__VER__ > 300)

  extern void OSCtxSw_ISR(void);
  extern void OSTick_ISR(void);

  #pragma vector = 0x38  //0x2C
  __interrupt void OStickvector(void)
  {
    OSTick_ISR();
  }

  #pragma vector = 0x7E
  __interrupt void OSshedvector(void)
  {
    OSCtxSw_ISR();
  }
 #endif 
#endif
 
/**********************************************************/
