;****************************************************************
;*                       pC/OS 1.70b                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                      OS_Rabbit_a.asm                         *
;*      Rabbit 2000/3000 Specific Code (FAR MEMORY MODEL)       *
;*                      using SoftTools                         *
;****************************************************************
        PUBLIC $OSStartHighRdy
        PUBLIC $OSCtxSw
        PUBLIC $OSIntCtxSw
        PUBLIC $OSTickISR
        EXTERN $OSIntEnter
        EXTERN $OSIntExit
        EXTERN $OSSystemTick
        EXTERN _OSTCBCur
        EXTERN _OSTCBHighRdy
.DEFSEG FARCODE
.SEG    FARCODE

;****************************************************************
; The following macro is the assembly code to save the current
; task context
save_context    MACRO
        PUSH   IP                  ; Save current task's context
        PUSH   AF
        PUSH   BC
        PUSH   DE
        PUSH   HL
        EXX
        EX     AF,AF'
        PUSH   AF
        PUSH   BC
        PUSH   DE
        PUSH   HL
        PUSH   IX
        PUSH   IY
        LD     HL,20               ; Offset to IP on stack
        ADD    HL,SP
        RRC    (HL)                ; IPRES on IP on stack
        RRC    (HL)                ; to enable INT's after context-switch
ENDM

;****************************************************************
; The following macro is the assembly code to restore the current
; task context
restore_context MACRO
        POP    IY
        POP    IX
        POP    HL                  ; Load task's context
        POP    DE
        POP    BC
        POP    AF
        EXX
        EX     AF,AF'
        POP    HL
        POP    DE
        POP    BC
        POP    AF
        POP    IP
ENDM

;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
$OSStartHighRdy:
        LD     HL,(_OSTCBHighRdy)  ; Get highest prio. task
        LD     (_OSTCBCur),HL
        LD     HL,(HL)             ; Get ptr to top of stack
        LD     SP,HL
        restore_context
        LRET                       ; Run task

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
$OSCtxSw:
        save_context
        BOOL   HL
        LD     L,H
        ADD    HL,SP
        LD     IX,(_OSTCBCur)      ; Save stack ptr in TCB
        LD     (IX),HL
        LD     HL,(_OSTCBHighRdy)  ; Point to HI Prio. Task Rdy
        LD     (_OSTCBCur),HL      ; this is now current TCB
        LD     HL,(HL)             ; Get new task's stack ptr
        LD     SP,HL
        restore_context
        LRET                       ; Return to new task

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
$OSIntCtxSw:
        LD     HL,11               ; Ignore calls to OS_IntExit,
                                   ; OSIntCtxSw and locals.
        ADD    HL,SP
        LD     IX,(_OSTCBCur)      ; Save stack ptr in TCB
        LD     (IX),HL
        LD     HL,(_OSTCBHighRdy)  ; Point to HI Prio. Task Rdy
        LD     (_OSTCBCur),HL      ; this is now current TCB
        LD     HL,(HL)             ; Get new task's stack ptr
        LD     SP,HL
        restore_context
        LRET                       ; Return to new task

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
$OSTickISR:
        save_context
        LCALL  $OSIntEnter
        LCALL  $OSSystemTick
        LCALL  $OSIntExit
        restore_context
        LRET                       ; Return to interrupted/new task

END
