/*************************************************************
*
* Description:
*   SAMD21 - Example program using Keil / IAR
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define SAMD21_TAU        0
#define SAMD21_SPARKFUN   1
#define SAMD21_15x15      2

#define SAMD21_TYPE       SAMD21_TAU


#define PORTA         0                       // user-LED definition
#define LED_GROUP     PORTA
#if (SAMD21_TYPE == SAMD21_TAU)               // -- TAU board
 #define LED_PORT     PORT_PA27
 #define LED_PIN      PIN_PA27
#endif
#if (SAMD21_TYPE == SAMD21_SPARKFUN)          // -- Sparkfun
 #define LED_PORT     PORT_PA17
 #define LED_PIN      PIN_PA17
#endif
#if (SAMD21_TYPE == SAMD21_15x15)             // -- SAMD15x15
 #define LED_PORT     PORT_PA09
 #define LED_PIN      PIN_PA09
#endif

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
 OS_SEM LedSem;
 OS_TMR LedTimer;
 OS_TMR ROnceTimer1;
 OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

#ifndef GCLK_CLKCTRL_ID_DFLL48_Val
 #define   GCLK_CLKCTRL_ID_DFLL48_Val   0x0ul
#endif
#define   GCLK_CLKCTRL_ID_DFLL48      (GCLK_CLKCTRL_ID_DFLL48_Val << GCLK_CLKCTRL_ID_Pos)
void SysTick_init(void)
{
    SYSCTRL->XOSC32K.reg  = SYSCTRL_XOSC32K_ENABLE;
    SYSCTRL->XOSC32K.reg |= SYSCTRL_XOSC32K_STARTUP(5) | SYSCTRL_XOSC32K_AAMPEN | SYSCTRL_XOSC32K_EN32K | SYSCTRL_XOSC32K_XTALEN | SYSCTRL_XOSC32K_ENABLE;
    while((SYSCTRL->PCLKSR.reg & SYSCTRL_PCLKSR_XOSC32KRDY) == 0) __NOP();
    GCLK->GENDIV.reg  = GCLK_GENDIV_DIV(1) | GCLK_GENDIV_ID(1);
    GCLK->GENCTRL.reg = GCLK_GENCTRL_ID(1) | GCLK_GENCTRL_SRC_XOSC32K | GCLK_GENCTRL_GENEN;
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_GEN_GCLK1 | GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_ID(0) ;
    SYSCTRL->DFLLCTRL.reg  = SYSCTRL_DFLLCTRL_ENABLE;
    SYSCTRL->DFLLMUL.reg   = SYSCTRL_DFLLMUL_CSTEP(5) | SYSCTRL_DFLLMUL_FSTEP(10); 	
    SYSCTRL->DFLLMUL.reg  |= SYSCTRL_DFLLMUL_MUL(1465);    // 1465 * 32768 is approx 48MHZ
    SYSCTRL->DFLLCTRL.reg |= SYSCTRL_DFLLCTRL_MODE;
    while((SYSCTRL->PCLKSR.reg & SYSCTRL_PCLKSR_DFLLLCKC) == 0) __NOP();
    NVMCTRL->CTRLB.reg |= NVMCTRL_CTRLB_RWS(1);
    GCLK->GENDIV.reg  = GCLK_GENDIV_DIV(1) | GCLK_GENDIV_ID(0);
    GCLK->GENCTRL.reg = GCLK_GENCTRL_GENEN | GCLK_GENCTRL_SRC_DFLL48M | GCLK_GENCTRL_ID(0);
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN_GCLK0 | GCLK_CLKCTRL_ID_DFLL48;
    // ---------------------
    SystemCoreClockUpdate();
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
}

void SysTick_update(void)
{
}

void LED_init(void)
{
    PORT->Group[LED_GROUP].DIRSET.reg = LED_PORT;
    PORT->Group[LED_GROUP].OUTSET.reg = LED_PORT;
}

void LED_toggle(void)
{
    PORT->Group[LED_GROUP].OUTTGL.reg = LED_PORT;
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);
#if OS_STK_CHECK_EN
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], &Stk1, 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], &Stk2, 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], &Stk3, 12);
#else
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], 12);
#endif
    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical Cortex-M0 pC/OS ISR
{
    OS_IntEnter();

    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
    OS_IntExit();
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((OS_MEM OS_HUGE *)ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;

    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif

    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
