/*
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * ----------------------------------------------------------------------
 *
 * $Date:        15. October 2016
 * $Revision:    1.1.0
 *
 * Project:      TrustZone for ARMv8-M
 * Title:        Code template for secure main function
 *
 *---------------------------------------------------------------------------*/
 
/* Use CMSE intrinsics */
#include <arm_cmse.h>

#include <stdio.h>
#include "sam.h"                        /* Device header */
 
/* TZ_START_NS: Start address of non-secure application */
#ifndef TZ_START_NS
#define TZ_START_NS (0x00001000U)
#endif
 
/* typedef for non-secure callback functions */
typedef void (*funcptr_void) (void) __attribute__((cmse_nonsecure_call));
 
/*
	The Secure Application:
	* Sets the non-secure main stack (MSP_NS) 
	* Gets the non-secure reset handler
	* Jumps to the non-secure software application
*/

/* Secure main() */
int main(void) {
  funcptr_void NonSecure_ResetHandler;
 
  /* Add user setup code for secure part here*/
    
        /* (1) set clock up to 16MHz */
        PM->INTFLAG.reg = PM_INTFLAG_PLRDY;
        PM->PLCFG.reg = PM_PLCFG_PLSEL_PL2;
        while (0 == PM->INTFLAG.bit.PLRDY);
        OSCCTRL->OSC16MCTRL.reg = OSCCTRL_OSC16MCTRL_ENABLE | OSCCTRL_OSC16MCTRL_FSEL_16;
        
        /* (2) Allocate PA27 (LED pin), PA22 & PA23 (I2C) , PA16..19 (SPI) to Non Secure world */
        PORT_SEC->Group[0].NONSEC.reg = (PORT_PA27 | PORT_PA22 | PORT_PA23 | PORT_PA16 | PORT_PA17 | PORT_PA18 | PORT_PA19);

        /* (3) Setup all required 'init' for unsecure SERCOM1 access in Non Secure world (UROW must allow unsecure access SERCOM1) for SPI */
        MCLK->APBCMASK.reg |= MCLK_APBCMASK_SERCOM1;
        GCLK->PCHCTRL[SERCOM1_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN(0) | GCLK_PCHCTRL_CHEN;
        while (0 == (GCLK->PCHCTRL[SERCOM1_GCLK_ID_CORE].reg & GCLK_PCHCTRL_CHEN));
 
        /* (4) Setup all required 'init' for unsecure SERCOM0 access in Non Secure world (UROW must allow unsecure access SERCOM0) for I2C */
        MCLK->APBCMASK.reg |= MCLK_APBCMASK_SERCOM0;
        GCLK->PCHCTRL[SERCOM0_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN(0) | GCLK_PCHCTRL_CHEN;
        while(!(GCLK->PCHCTRL[SERCOM0_GCLK_ID_CORE].bit.CHEN));
    
  /* Set non-secure main stack (MSP_NS) */
  __TZ_set_MSP_NS(*((uint32_t *)(TZ_START_NS)));
    
  /* Get non-secure reset handler */
  NonSecure_ResetHandler = (funcptr_void)(*((uint32_t *)((TZ_START_NS) + 4U)));
 
  /* Start non-secure state software application */
  NonSecure_ResetHandler();
 
  /* Non-secure software does not return, this code is not executed */
  while (1) {
    __NOP();
  }
}
