/*************************************************************
*
* Description:
*   TMS570 - mini VIM support for pC/OS RTOS kernel
*            to get the central IRQ-entry functional
*   This is a minimalistic VIM implementation w/o the TMS570
*   system will not support IRQ/FIQ event system
*
* Author[s]:
*   Frank Goetze
**************************************************************/

typedef void (*VecFuncPTR)(void);

typedef volatile struct {
  VecFuncPTR ISR[97]; 
} miniVIM_ram_t;

typedef volatile struct {
    unsigned PARFLG;            /* InterruptVectorTableParityFlagRegister */
    unsigned PARCTL;            /* InterruptVectorTableParityControlRegister */
    unsigned ADDERR;            /* AddressParityErrorRegister */
    unsigned FBPARERR;          /* Fall-BackAddressParityErrorRegister */
    unsigned reserved1 [0x4/4];
    unsigned IRQINDEX;          /* IRQIndexOffsetVectorRegister */
    unsigned FIQINDEX;          /* FIQIndexOffsetVectorRegister */
    unsigned reserved2 [0x8/4];
    unsigned FIRQPR[3];         /* FIQ/IRQProgramControlRegister0 */
    unsigned reserved3 [0x4/4];
    unsigned _INTREQ[3];        /* PendingInterruptReadLocationRegister0 */
    unsigned reserved4 [0x4/4];
    unsigned REQENASET[3];      /* InterruptEnableSetRegister0 */
    unsigned reserved5 [0x4/4];
    unsigned REQENACLR[3];      /* InterruptEnableClearRegister0 */
    unsigned reserved6 [0x4/4];
    unsigned WAKEENASET[3];     /* Wake-upEnableSetRegister0 */
    unsigned reserved7 [0x4/4];
    unsigned WAKEENACLR[3];     /* Wake-upEnableClearRegister0 */
    unsigned reserved8 [0x4/4];
    unsigned IRQVECREG;         /* IRQInterruptVectorRegister */
    unsigned FIQVECREG;         /* FIQInterruptVectorRegister */
    unsigned CAPEVT;            /* CaptureEventRegister */
    unsigned reserved9 [0x4/4];
    unsigned CHANCTRL [0x5c/4]; /* VIM Interrupt Control Register (PARSER ERROR) */
} miniVIM_t;

void miniVIM_dummy_interrupt(void);
void miniVIM_set_vector(unsigned vector_nr, VecFuncPTR fkt);
void miniVIM_enable_int(unsigned vector_nr);
void miniVIM_disable_int(unsigned vector_nr);
void miniVIM_Init(void);

/*************************************************************/
