/*************************************************************
*
* Description:
*   TMS570 - Example program using Keil / IAR
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define PLL_CPU_CLK  160000000

#define LED_PORT     gioPORTA 
#define LED_PIN      2

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
 OS_SEM LedSem;
 OS_TMR LedTimer;
 OS_TMR ROnceTimer1;
 OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

/***************************************************************/
#include "miniVIM.h"
void OSTickISR(void);

extern void systemInit(void);
void SystemInit(void)
{
    systemInit();
}

RTI_ST    *rtiREG = (RTI_ST *) RTI;
void SysTick_init(void)
{
    miniVIM_Init();
    miniVIM_set_vector(2, OSTickISR);
    miniVIM_enable_int(2);                   

    rtiREG->RTIGCTRL_UN.RTIGCTRL_UL       = 0x00000000U;   
    rtiREG->RTITBCTRL_UN.RTITBCTRL_UL     = 0x00000000U;   
    rtiREG->RTICOMPCTRL_UN.RTICOMPCTRL_UL = 0x00000010U;   
    rtiREG->RTIUC0_UL                     = 0x00000000U;   
    rtiREG->RTIFRC0_UL                    = 0x00000000U;   
    rtiREG->RTICPUC0_UL                   = 0x00000001U;   
    rtiREG->RTICOMP0_UL                   = PLL_CPU_CLK / 2 / OS_TIMER_TICKS_PER_SEC;   
    rtiREG->RTIUDCP0_UL                   = PLL_CPU_CLK / 2 / OS_TIMER_TICKS_PER_SEC;   
    rtiREG->RTIINTFLAG_UN.RTIINTFLAG_UL   = 0x0007000FU;   
    rtiREG->RTICLEARINT_UN.RTICLEARINT_UL = 0x00070F0FU;   
    rtiREG->RTISETINT_UN.RTISETINT_UL     = 0x00000001U;   
    rtiREG->RTIGCTRL_UN.RTIGCTRL_UL      |= 0x00000001U;   
}

void SysTick_update(void)
{
    rtiREG->RTIINTFLAG_UN.RTIINTFLAG_UL   = 0x00000001U;
}

void LED_init(void)
{
    gioREG->GCR0      = 1;   
    gioREG->INTENACLR = 0xFF;   
    gioREG->LVLCLR    = 0xFF;
    LED_PORT->DOUT    = (0 << LED_PIN);
    LED_PORT->DIR     = (1 << LED_PIN);
    LED_PORT->PDR     = (0 << LED_PIN);
    LED_PORT->PSL     = (0 << LED_PIN);
    LED_PORT->PULDIS  = (0 << LED_PIN);    
}

void LED_toggle(void)
{
    if (LED_PORT->DIN & (1 << LED_PIN)) LED_PORT->DCLR = 1 << LED_PIN;
    else                                LED_PORT->DSET = 1 << LED_PIN;
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);
#if OS_STK_CHECK_EN
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], &Stk1, 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], &Stk2, 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], &Stk3, 12);
#else
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], 12);
#endif
    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical Cortex-R4 pC/OS ISR (behind central IRQ-entry & miniVIM dispatcher)
{
    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((OS_MEM OS_HUGE *)ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;
 
    // ---- HW specific ----
    LED_init();
    // ---------------------
    
LED_toggle();

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif

    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
