/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                       OS_V850_c.c                        *
*                Renesas V850 Specific Code                *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U32 *stk;
    U08  err;
#if USE_FPU
    U08  i;
#endif

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk      = (U32 *)pstk;                               /* decrement first !                    */
        *--stk   = (U32) pstk;                                /* task stack pointer                   */
        *--stk   = (U32) 0x00000000;                          /* Task PSW = Interrupts disabled       */
        *--stk   = (U32) task;                                /* Task's PC code                       */
        *--stk   = (U32) 0x31313131;                          /* r31 Link pointer                     */
        *--stk   = (U32) 0x30303030;                          /* r30                                  */
        *--stk   = (U32) 0x29292929;                          /* r29                                  */
        *--stk   = (U32) 0x28282828;                          /* r28 Frame pointer/permanent          */
        *--stk   = (U32) 0x27272727;                          /* r27                                  */
        *--stk   = (U32) 0x26262626;                          /* r26                                  */
        *--stk   = (U32) 0x25252525;                          /* r25                                  */
        *--stk   = (U32) 0x24242424;                          /* r24                                  */
        *--stk   = (U32) 0x23232323;                          /* r23                                  */
        *--stk   = (U32) 0x22222222;                          /* r22                                  */
        *--stk   = (U32) 0x21212121;                          /* r21                                  */
        *--stk   = (U32) 0x20202020;                          /* r20                                  */
        *--stk   = (U32) 0x19191919;                          /* r19                                  */
        *--stk   = (U32) 0x18181818;                          /* r18                                  */
        *--stk   = (U32) 0x17171717;                          /* r17                                  */
        *--stk   = (U32) 0x16161616;                          /* r16                                  */
        *--stk   = (U32) 0x15151515;                          /* r15                                  */
        *--stk   = (U32) 0x14141414;                          /* r14                                  */
        *--stk   = (U32) 0x13131313;                          /* r13                                  */
        *--stk   = (U32) 0x12121212;                          /* r12                                  */
        *--stk   = (U32) 0x11111111;                          /* r11                                  */
        *--stk   = (U32) 0x10101010;                          /* r10 Function return value/temporary  */
        *--stk   = (U32) 0x09090909;                          /* r9 Parameters                        */
        *--stk   = (U32) 0x08080808;                          /* r8 Parameters                        */
        *--stk   = (U32) 0x07070707;                          /* r7 Parameters                        */
        *--stk   = (U32) 0x06060606;                          /* r6 Parameters                        */
        *--stk   = (U32) 0x05050505;                          /* r5 Temporary or Rosda base           */
        *--stk   = (U32) 0x02020202;                          /* r2 Temporary                         */
        *--stk   = (U32) Tdata;                               /* r1 data / Address generation in compiler and assembler */
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

/**********************************************************/
