;****************************************************************
;*                       pC/OS 1.70b                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                      OS_Z180_a.asm                           *
;*           Z180 Specific Code (BANKED MEMORY MODEL)           *
;*                     using SoftTools                          *
;****************************************************************
    RSEG    RCODE(0)
            
        PUBLIC  OSReset
        PUBLIC  OSIntVecTbl
        PUBLIC  OSStartHighRdy
        PUBLIC  OSCtxSw
        PUBLIC  OSIntCtxSw
        PUBLIC  OSTickISR
        EXTERN  OSIntEnter
        EXTERN  OSIntExit
        EXTERN  OSSystemTick
        EXTERN  OSTCBCur
        EXTERN  OSTCBHighRdy
        EXTERN  ?BANK_CALL_DIRECT_L08

    ASEG
    
OSReset:
        ORG     00H
        
        ORG     08H
        
        ORG     10H
        
        ORG     18H
        
        ORG     20H
        JP      OSCtxSw            ; jump to context-switch
        ORG     28H
        
        ORG     30H
        
        ORG     38H
        
        
    COMMON  INTVEC

OSIntVecTbl:
        DEFW    0                  ; INT1
        DEFW    0                  ; INT2
        DEFW    OSTickISR          ; TIMER0
        DEFW    0                  ; TIMER1
        DEFW    0                  ; DMA0
        DEFW    0                  ; DMA1
        DEFW    0                  ; CSIO
        DEFW    0                  ; ASCI0
        DEFW    0                  ; ASCI1

    RSEG    RCODE
;****************************************************************
; The following macro is the assembly code to save the current
; task context
save_context    MACRO
        PUSH   AF                  ; Save current task's context
        PUSH   BC
        PUSH   DE
        PUSH   HL                                 
        EX     AF,AF'                             
        EXX 
        PUSH   AF                                 
        PUSH   BC
        PUSH   DE
        PUSH   HL                                 
        EX     AF,AF'                             
        EXX 
        PUSH   IX
        PUSH   IY
ENDM

;****************************************************************
; The following macro is the assembly code to restore the current
; task context
restore_context MACRO
        POP    IY                  ; Pop task's registers
        POP    IX
        EX     AF,AF' 
        EXX 
        POP    HL 
        POP    DE 
        POP    BC 
        POP    AF
        EX     AF,AF' 
        EXX 
        POP    HL 
        POP    DE 
        POP    BC 
        POP    AF
ENDM

;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
OSStartHighRdy:
        LD     HL,(OSTCBHighRdy)   ; Get highest prio. task
        LD     (OSTCBCur),HL 
        INC    HL      
        LD     H,(HL)  
        LD     L,E     
        LD     SP,HL   
        POP    AF                  ; Restore Interrupted task's Bank
        OUT0   (039H),A
        restore_context
        RET                        ; Run task

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
OSCtxSw:
        save_context
        IN0    A,(039H)            ; Save Interrupted task's Bank
        PUSH   AF
        LD     HL,0
        ADD    HL,SP           
        EX     DE,HL
        LD     HL,(OSTCBCur)       ; Save stack ptr in TCB
        LD     (HL),E          
        INC    HL              
        LD     (HL),D          
        LD     HL,(OSTCBHighRdy)   ; Point to HI Prio. Task Rdy
        LD     (OSTCBCur),HL       ; this is now current TCB
        LD     E,(HL)              ; Get new task's stack ptr
        INC    HL      
        LD     H,(HL)  
        LD     L,E     
        LD     SP,HL   
        POP    AF                  ; Restore Interrupted task's Bank
        OUT0   (039H),A                           
        restore_context
        RET                        ; Return to new task

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
OSIntCtxSw:
        LD     HL,10               ; Ignore calls to OS_IntExit,
                                   ; OSIntCtxSw and locals.
        ADD    HL,SP                               
        EX     DE,HL
        LD     HL,(OSTCBCur)       ; Save stack ptr in TCB                      
        LD     (HL),E                              
        INC    HL                                  
        LD     (HL),D                              
        LD     HL,(OSTCBHighRdy)   ; Point to HI Prio. Task Rdy
        LD     (OSTCBCur),HL       ; this is now current TCB
        LD     E,(HL)              ; Get new task's stack ptr
        INC    HL      
        LD     H,(HL)  
        LD     L,E     
        LD     SP,HL   
        POP    AF                  ; Restore Interrupted task's Bank
        OUT0   (039H),A                           
        restore_context
        RET                        ; Return to new task

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
OSTickISR:
        save_context
        IN0    A,(039H)            ; Save Interrupted task's Bank
        PUSH   AF
;-------
        IN     A,(010H)            ; read control and data ...
        IN     A,(00CH)			   ; ... to clear INT
;-------
        LD     A,0
        LD     HL,OSIntEnter
        CALL   ?BANK_CALL_DIRECT_L08
        LD     HL,OSSystemTick
        CALL   ?BANK_CALL_DIRECT_L08
        LD     HL,OSIntExit
        CALL   ?BANK_CALL_DIRECT_L08
        POP    AF                  ; Restore Interrupted task's Bank
        OUT0   (039H),A                           
        restore_context
        RET                        ; Return to interrupted/new task

END
