/***********************************************************
*                       pC/OS 1.88c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_sem.c                         *
*                   SEMAPHORE SERVICE                      *
***********************************************************/

#if OS_SEM_EN
/*
************************************************************
*                   INITIALIZE SEMAPHORE                   *
************************************************************
*/
U08 OS_SemInit(OS_SEM *psem, U16 cnt) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08 i;

    OS_ENTER_CRITICAL();
    psem->OSSemCnt = cnt;
    psem->OSSemGrp = 0x00;
    for (i = 0; i < OS_PRIO_TBL; i++)
        psem->OSSemTbl[i] = 0x00;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                     PEND ON SEMAPHORE                    *
************************************************************
*/
U08 OS_SemPend(OS_SEM *psem, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity;

    OS_ENTER_CRITICAL();
    if (psem->OSSemCnt > 0) {
        psem->OSSemCnt--;
        OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    }
    if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
        OS_EXIT_CRITICAL();
        return (OS_SEM_NODATA);
    }
#if OS_TASK_DESTROY_EN
    OSTCBCur->OSTCBipc   = psem;
#endif
    OSTCBCur->OSTCBStat |= OS_STAT_SEM;
    OSTCBCur->OSTCBDly   = timeout;
    y                    = OSTCBCur->OSTCBPrio >> 3;
    x                    = OSTCBCur->OSTCBPrio & 0x07;
    bity                 = OSMapTbl[y];
    bitx                 = OSMapTbl[x];
    if ((OSRdyTbl[y] &= ~(bitx)) == 0)
        OSRdyGrp &= ~(bity);
    psem->OSSemTbl[y] |= bitx;
    psem->OSSemGrp    |= bity;
    OS_EXIT_CRITICAL();
    OSSched();
    OS_ENTER_CRITICAL();
    if ((OSTCBCur->OSTCBStat & OS_STAT_SEM) && (psem->OSSemCnt == 0)) {
        if ((U08)(psem->OSSemTbl[y] &= ~(bitx)) == 0)
            psem->OSSemGrp &= ~(bity);
        OSTCBCur->OSTCBStat = OS_STAT_RDY;
        OS_EXIT_CRITICAL();
        return (OS_TIMEOUT);
    }
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                    POST TO A SEMAPHORE                   *
************************************************************
*/
U08 OS_SemPost(OS_SEM *psem) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (psem->OSSemGrp != 0) {
        y    = OSUnMapTbl[psem->OSSemGrp];
        x    = OSUnMapTbl[psem->OSSemTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(psem->OSSemTbl[y] &= ~(bitx)) == 0)
            psem->OSSemGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_SEM);
        OSRdyGrp                   |=  bity;
        OSRdyTbl[y]                |=  bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    if (psem->OSSemCnt < (U16)(-1)) {
        psem->OSSemCnt++;
        OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_SEM_OVF);
}

#if OS_SEM_EXT_EN
/*
************************************************************
*                     ACCEPT A SEMAPHORE                   *
************************************************************
*/
U08 OS_SemAccept(OS_SEM *psem, U16 *cnt, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity;

    OS_ENTER_CRITICAL();
    if ((*cnt = psem->OSSemCnt) > 0) {
        OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    }
    if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
        OS_EXIT_CRITICAL();
        return (OS_SEM_NODATA);
    }
#if OS_TASK_DESTROY_EN
    OSTCBCur->OSTCBipc   = psem;
#endif
    OSTCBCur->OSTCBStat |= OS_STAT_SEM;
    OSTCBCur->OSTCBDly   = timeout;
    y                    = OSTCBCur->OSTCBPrio >> 3;
    x                    = OSTCBCur->OSTCBPrio & 0x07;
    bity                 = OSMapTbl[y];
    bitx                 = OSMapTbl[x];
    if ((OSRdyTbl[y] &= ~(bitx)) == 0)
        OSRdyGrp &= ~(bity);
    psem->OSSemTbl[y] |= bitx;
    psem->OSSemGrp    |= bity;
    OS_EXIT_CRITICAL();
    OSSched();
    OS_ENTER_CRITICAL();
    if ((OSTCBCur->OSTCBStat & OS_STAT_SEM) && (psem->OSSemCnt == 0)) {
        if ((U08)(psem->OSSemTbl[y] &= ~(bitx)) == 0)
            psem->OSSemGrp &= ~(bity);
        OSTCBCur->OSTCBStat = OS_STAT_RDY;
        OS_EXIT_CRITICAL();
        return (OS_TIMEOUT);
    }
    *cnt = psem->OSSemCnt;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*           ABBORT HIGHEST PENDER ON A SEMAPHORE           *
************************************************************
*/
U08 OS_SemPendAbbort(OS_SEM *psem) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (psem->OSSemGrp != 0) {
        y    = OSUnMapTbl[psem->OSSemGrp];
        x    = OSUnMapTbl[psem->OSSemTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(psem->OSSemTbl[y] &= ~(bitx)) == 0)
            psem->OSSemGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_SEM);
        OSRdyGrp                   |=  bity;
        OSRdyTbl[y]                |=  bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                     CLEAR A SEMAPHORE                    *
************************************************************
*/
U08 OS_SemClear(OS_SEM *psem) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    psem->OSSemCnt = 0;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
#endif /* OS_SEM_EXT_EN */

#if OS_TASK_DESTROY_EN
/*
************************************************************
*               DELETE A TASK FROM A SEMAPHORE             *
************************************************************
*/
void OSSemDestroyTask(OS_SEM *psem, U08 prio)
{
    U08          x, y, bitx, bity;

    y      = prio >> 3;
    x      = prio & 0x07;
    bity   = OSMapTbl[y];
    bitx   = OSMapTbl[x];
    if ((U08)(psem->OSSemTbl[y] &= ~(bitx)) == 0)
        psem->OSSemGrp &= ~(bity);
    OSTCBPrioTbl[prio]->OSTCBDly   =  0;
    OSTCBPrioTbl[prio]->OSTCBStat &= ~(OS_STAT_SEM);
    OSRdyGrp                      |=  bity;
    OSRdyTbl[y]                   |=  bitx;
}
#endif /* OS_TASK_DESTROY_EN */
#endif /* OS_SEM_EN */

/************************* END ****************************/


